#include "PlayerIncludes.h"

void Player::displayFileInfo()
{
   if (selectedFile == "")
      return;

   MediaInfoBase mdf(this, tr("File info"), TRUE, 0);
   QFileInfo fs(selectedFile);
   mdf.setCaption(tr("File info") + " - " + fs.fileName());

   mdf.title_label->setAlignment(Qt::AlignLeft | Qt::AlignVCenter | Qt::ExpandTabs | Qt::WordBreak);
   mdf.album_label->setAlignment(Qt::AlignLeft | Qt::AlignVCenter | Qt::ExpandTabs | Qt::WordBreak);
   mdf.artist_label->setAlignment(Qt::AlignLeft | Qt::AlignVCenter | Qt::ExpandTabs | Qt::WordBreak);
   mdf.comment_label->setAlignment(Qt::AlignLeft | Qt::AlignVCenter | Qt::ExpandTabs | Qt::WordBreak);

   mdf.title_label->setText(minfo[selectedFile].Title);
   mdf.album_label->setText(minfo[selectedFile].Album);
   mdf.artist_label->setText(minfo[selectedFile].Artist);
   mdf.year_label->setText(minfo[selectedFile].Year);
   mdf.comment_label->setText(minfo[selectedFile].Comment);

   QString s_length = length(minfo[selectedFile].LENGTH);
   mdf.length_label->setText(s_length);

   QString size = QString::number(minfo[selectedFile].VIDEO_WIDTH) + "x" + QString::number(minfo[selectedFile].VIDEO_HEIGHT);
   mdf.size_label->setText(size);
   mdf.vbitrate_label->setText(QString::number(minfo[selectedFile].VIDEO_BITRATE));
   mdf.vformat_label->setText(minfo[selectedFile].VIDEO_FORMAT);
   mdf.fps_label->setText(QString::number(minfo[selectedFile].VIDEO_FPS));

   mdf.achans_label->setText(QString::number(minfo[selectedFile].AUDIO_NCH));
   mdf.acodec_label->setText(minfo[selectedFile].AUDIO_CODEC);
   mdf.arate_label->setText(QString::number(minfo[selectedFile].AUDIO_RATE));
   mdf.abitrate_label->setText(QString::number(minfo[selectedFile].AUDIO_BITRATE));

   mdf.exec();
}

void Player::parseMediaInfo(const QString& file, const QString& strUtf8, const QString& strLocal)
{
   if (strLocal.startsWith("Title:")) {
       minfo[file].Title = strLocal.mid(6);
   } else if (strUtf8.upper().startsWith("OGGVORBISCOMMENT: TITLE=")) {
       minfo[file].Title = strUtf8.mid(24);
   } else if (strLocal.startsWith("name:")) {
       minfo[file].Title = strLocal.mid(5);
   } else if (strLocal.startsWith("Artist:")) {
       minfo[file].Artist = strLocal.mid(7);
   } else if (strUtf8.upper().startsWith("OGGVORBISCOMMENT: ARTIST=")) {
       minfo[file].Artist = strUtf8.mid(25);
   } else if (strLocal.startsWith("author:")) {
       minfo[file].Artist = strLocal.mid(7);
   } else if (strLocal.startsWith("Album:")) {
       minfo[file].Album = strLocal.mid(6);
   } else if (strUtf8.upper().startsWith("OGGVORBISCOMMENT: ALBUM=")) {
       minfo[file].Album = strUtf8.mid(24);
    } else if (strLocal.startsWith("Year:")) {
       minfo[file].Year = strLocal.mid(5);
    } else if (strUtf8.upper().startsWith("OGGVORBISCOMMENT: DATE=")) {
       minfo[file].Year = strUtf8.mid(23);
   } else if (strLocal.startsWith("Comment:")) {
       minfo[file].Comment = strLocal.mid(8);
   } else if (strLocal.startsWith("comments:")) {
       minfo[file].Comment = strLocal.mid(8);
   } else if (strUtf8.upper().startsWith("OGGVORBISCOMMENT: DESCRIPTION=")) {
       minfo[file].Comment = strUtf8.mid(30);
   } else if (strLocal.startsWith("Track:")) {
       minfo[file].Track = strLocal.mid(6);
   } else if (strUtf8.upper().startsWith("OGGVORBISCOMMENT: TRACKNUMBER=")) {
       minfo[file].Track = strUtf8.mid(31);
   } else if (strLocal.startsWith("Genre:")) {
       minfo[file].Genre = strLocal.mid(6);
   } else if (strUtf8.startsWith("ID_FILENAME=")) {
       minfo[file].FILENAME = strUtf8.mid(12);
   } else if (strUtf8.startsWith("ID_AUDIO_CODEC=")) {
       minfo[file].AUDIO_CODEC = strUtf8.mid(15);
   } else if (strUtf8.startsWith( "ID_AUDIO_FORMAT=")) {
       minfo[file].AUDIO_FORMAT = strUtf8.mid(16).toInt();
   } else if (strUtf8.startsWith( "ID_AUDIO_BITRATE=")) {
       minfo[file].AUDIO_BITRATE = strUtf8.mid(17).toInt();
   } else if (strUtf8.startsWith( "ID_AUDIO_RATE=")) {
       minfo[file].AUDIO_RATE = strUtf8.mid(14).toInt();
   } else if (strUtf8.startsWith( "ID_AUDIO_NCH=")) {
       minfo[file].AUDIO_NCH = strUtf8.mid(13).toInt();
   } else if (strUtf8.startsWith( "ID_LENGTH=")) {
       minfo[file].LENGTH = strUtf8.mid(10).toInt();
//     if (minfo[file].FILENAME.contains(".mp3"))
//       minfo[file].LENGTH = (minfo[file].LENGTH / 100) * 60 + minfo[file].LENGTH % 100;
   } else if (strUtf8.startsWith( "ID_VIDEO_FORMAT=")) {
       minfo[file].VIDEO_FORMAT = strUtf8.mid(16);
   } else if (strUtf8.startsWith( "ID_VIDEO_BITRATE=")) {
       minfo[file].VIDEO_BITRATE = strUtf8.mid(17).toInt();
   } else if (strUtf8.startsWith( "ID_VIDEO_WIDTH=")) {
       minfo[file].VIDEO_WIDTH = strUtf8.mid(15).toInt();
   } else if (strUtf8.startsWith( "ID_VIDEO_HEIGHT=")) {
       minfo[file].VIDEO_HEIGHT = strUtf8.mid(16).toInt();
   } else if (strUtf8.startsWith( "ID_VIDEO_FPS=")) {
       minfo[file].VIDEO_FPS = strUtf8.mid(13).toFloat();
   } else if (strUtf8.startsWith( "ID_VIDEO_ASPECT=")) {
       minfo[file].VIDEO_ASPECT = strUtf8.mid(16).toInt();
   }

   //qDebug("parseMediaInfo(): " + strUtf8.utf8());
}

