#include "PlayerIncludes.h"

void Player::fileMenu(int item)
{
   switch (item)
   {
      case FILE_OPEN:
      {
         if (isPlaying)
             sendCommand("quit\n");

         stack->raiseWidget(slf);
         slf->setFocus();
         setupButtons();
         return;
      }
       case FILE_OPEN_URL:
       {
         if (isPlaying)
             sendCommand("quit\n");

         InputDialog* urlDlg = new InputDialog(this, tr("Open URL..."),TRUE, 0);

         urlDlg->setHistory(kinoPrefs.s_url_history);
         urlDlg->exec();
         if( urlDlg->result() == 1 )
         {
             setSelectedFile(urlDlg->getInputText());

            if (kinoPrefs.f_debug)
                qDebug(urlDlg->getInputText());

             QString url_history = urlDlg->getHistory();
             if (!url_history.contains(selectedFile))
               url_history = selectedFile + "," + url_history;

             kinoPrefs.s_url_history = url_history;
             saveSettings();

             //contents->playlist->addToPlaylist(selectedFile);
             addUrl(selectedFile);
         }

         delete urlDlg;
         return;
       }
       case FILE_SAVE_PLAYLIST:
       {
         if (isPlaying)
             sendCommand("quit\n");

         FileDialog saveAs(FileDialog::MODE_SAVE, "*.m3u", this);
         saveAs.setDefaultName("playlist");
         saveAs.setComplementExt("m3u");
         saveAs.setFixedSize(this->size());

//       saveAs.show();
         if (saveAs.exec())
         {
            QString file = saveAs.getFilePath();
            savePlaylist(file);
         }
         return;
       }
       case FILE_INFO:
       {
         displayFileInfo();
         return;
       }
       case FILE_PREFS:
       {
         if (isPlaying)
             sendCommand("quit\n");

         bool save_remote = kinoPrefs.f_remote;
         KinoSettings *kinoset = new KinoSettings(this, tr("Preferences..."),TRUE,0);
         kinoset->setSettings(kinoPrefs);
         if (kinoset->exec() == 1)
         {
            // save URL history
            QString url_history = kinoPrefs.s_url_history;

             kinoPrefs = kinoset->getSettings();
             kinoPrefs.s_url_history = url_history;

             saveSettings();
             loadSettings();

             // update font
             contents->screen->setFont(QFont(kinoPrefs.s_font, kinoPrefs.s_fontSize, QFont::Bold));

             delete filter;
             filter = new FileSelectionFilter(kinoPrefs.s_types, this);
             slf->setFilter(filter);

            if (save_remote != kinoPrefs.f_remote)
            {
               int result = QMessageBox::warning(this, tr("Kino 2"), tr("The changes you made requrie Qtopia restart to take effect.\nDo you want to restart Qtopia now?"),
                  QMessageBox::Yes | QMessageBox::Default,
                  QMessageBox::No | QMessageBox::Escape);

               if (result == QMessageBox::Yes)
                  QCopEnvelope e("QPE/System", "restart()");
            }
         }
         return;
      }
    }
}

void Player::viewMenu(int item)
{
   switch(item)
   {
       case VIEW_SCALE:
       {
         m_viewMenu->setItemChecked(0, (useScale = useScale?false:true));
         return;
       }
       case VIEW_LCD_OFF:
       {
         lcdOff();
         //m_viewMenu->setItemChecked(1, (useFullscreen = useFullscreen?false:true));
         return;
       }
       case VIEW_PLAYLIST:
       {
         showPlaylist(!m_viewMenu->isItemChecked(VIEW_PLAYLIST));
         return;
       }
       case VIEW_SLIDER:
       {
         showSlider(!m_viewMenu->isItemChecked(VIEW_SLIDER));
         return;
       }
       case VIEW_ALBUM_COVER:
       {
         showAlbumCover(!m_viewMenu->isItemChecked(VIEW_ALBUM_COVER));
         return;
       }
   }
}

void Player::helpMenu(int item)
{
    if (item == 0)
    {
      QString str = tr("Kino 2 v" KINO_VERSION "\n"
           "MPlayer Qtopia GUI\n"
           "(c) Cacko Team, 2003-2005\n"
           "http://cacko.biz/kino2/\n");

      QMessageBox::information(this, tr("Kino 2"), str);
    }
}

void Player::columnsMenu(int item)
{
   bool show = !m_columnsMenu->isItemChecked(item);
   m_columnsMenu->setItemChecked(item, show);

   int column = m_columnsMenu->indexOf(item) + 2;
   if (show)
   {
      contents->playlist->setColumnWidthMode(column, QListView::Maximum);
      contents->playlist->setColumnWidth(column, 100);
   }
   else
   {
      contents->playlist->setColumnWidthMode(column, QListView::Manual);
      contents->playlist->setColumnWidth(column, 0);
   }

   contents->playlist->triggerUpdate();
}
