#include "PlayerIncludes.h"

QStringList Player::loadCoverImages(const QDir& folder)
{
   if (kinoPrefs.f_debug)
      qDebug("Player::loadCoverImages(): " + folder.absPath());

   QStringList images = folder.entryList("*.jpg;*.png;*.gif");
   return images;
}

void Player::displayCoverImage()
{
   // pick a random image
   if (!coverImages.isEmpty())
   {
      imageIndex++;
      if (imageIndex >= coverImages.count())
         imageIndex = 0;

      QFileInfo fi(selectedFile);
      QDir dir(fi.dirPath(true));

      if (kinoPrefs.f_debug)
      {
         qDebug(fi.absFilePath());
         qDebug(dir.absPath());
      }

      QString path = dir.absPath().append("/").append(coverImages[imageIndex]);

      if (kinoPrefs.f_debug)
         qDebug(path);

      QPixmap pm(path);

      if (!pm.isNull())
         contents->albumCover->setPixmap(pm);
      else
         contents->albumCover->setText(tr("No Cover Image"));

   }
}

void Player::showAlbumCover(bool visible)
{
   contents->setUpdatesEnabled(false);
   if (visible)
   {
      if (!m_viewMenu->isItemChecked(VIEW_PLAYLIST))
      {
         contents->albumCover->show();
      }
   }
   else
   {
      contents->albumCover->hide();
   }

   m_viewMenu->setItemChecked(VIEW_ALBUM_COVER, visible);
   contents->updateLayout(orientation, visible);

   contents->setUpdatesEnabled(true);
   setupButtons();
}


bool Player::addUrl(const QString& url)
{
    isAddToPlaylist = true;
      identifyFile(url);
      return true;
}

bool Player::addFile(const QFileInfo& fileInfo)
{
    isAddToPlaylist = true;
    isSelectFirst = false;
      identifyFile(fileInfo.absFilePath());
      return true;
}

bool Player::addFolder(const QFileInfo& fileInfo)
{
   QDir dir(fileInfo.absFilePath());
   QStringList files  = dir.entryList(kinoPrefs.s_types);
   QStringList list;

   if (files.isEmpty())
      return false;

   coverImages = loadCoverImages(dir);

   for (QStringList::Iterator it = files.begin(); it != files.end(); ++it)
   {
      if (!(*it).contains(".m3u") && !(*it).contains(".pls"))
         list << fileInfo.filePath() + "/" + (*it);
   }

   if (list.count() > 0)
   {
       isAddToPlaylist = true;
       isSelectFirst = true;
         identifyFile(list);
      return true;
   }

   return false;
}

bool Player::addPlaylist(const QFileInfo& fileInfo)
{
   QFile file(fileInfo.absFilePath());
   if (!file.open(IO_ReadOnly))
      return false;

   QDir dir(fileInfo.dirPath(true));
   coverImages = loadCoverImages(dir);

   QTextCodec* codec = QTextCodec::codecForName(kinoPrefs.s_enc_mp3);
   QStringList files;
   char buffer[256];
   int buflen = 0;

   while((buflen = file.readLine(buffer, 255)) != -1)
   {
      QString str = QString::fromLocal8Bit(buffer, buflen);
      if (codec != 0)
         str = codec->toUnicode(buffer, buflen);

      if (str[str.length() - 1] == '\n')
         str.truncate(str.length() - 1);

      QFileInfo fi;
      if (fileInfo.extension() == "m3u")
      {
         if (str.startsWith("#"))
            continue;

         fi.setFile(str);
      }
      else if (fileInfo.extension() == "pls")
      {
         if (!str.startsWith("File"))
            continue;

         str = str.mid(str.find("=") + 1);
         fi.setFile(str);
      }

      if (str.startsWith("http://"))
      {
         if (kinoPrefs.f_debug)
            qDebug("Adding URL: " + str);

         files << str;
         continue;
      }

      if (!fi.exists())
         fi.setFile(fileInfo.dirPath() + "/" + str);

      if (!fi.exists())
         continue;

      if (kinoPrefs.f_debug)
         qDebug("Adding file: " + fi.absFilePath());

      files << fi.absFilePath();
   }

   if (files.count() > 0)
   {
       isAddToPlaylist = true;
       isSelectFirst = true;
       identifyFile(files);
       return true;
   }

   return false;
}

void Player::savePlaylist(const QString& file)
{
    QString list;
   QTextCodec* codec = QTextCodec::codecForName(kinoPrefs.s_enc_mp3);

    QFile f(file);
    f.open(IO_WriteOnly | IO_Truncate);
    if (f.isOpen())
    {
       QListViewItemIterator it(contents->playlist);
       for(; it.current(); it++)
       {
         QString name = it.current()->text(0) + "\n";
         if (codec != 0)
            name = codec->fromUnicode(name);
         else
            name = name.local8Bit();

          f.writeBlock(name, name.length());
       }
       f.close();
    }
}

bool Player::addToPlaylist()
{
   if (!addToPls->isEnabled())
      return false;

   QString current = slf->currentFile();
   QFileInfo fileInfo(current);
   if (kinoPrefs.f_debug)
      qDebug("Player::addToPlaylist() - " + current);

   closeFileSelector = false;
   saveCurrentFile = current;
   return addToPlaylist(fileInfo);
}

bool Player::addAllToPlaylist()
{
   if (!addAllToPls->isEnabled())
      return false;

   QString current = slf->currentDir();
   QFileInfo fileInfo(current);
   if (kinoPrefs.f_debug)
      qDebug("Player::addAllToPlaylist() - " + current);

   closeFileSelector = false;
   saveCurrentFile = current;
   return addToPlaylist(fileInfo);
}

bool Player::addToPlaylist(const QFileInfo& fileInfo)
{
   bool added = false;

   if (fileInfo.filePath().startsWith("http://"))
   {
      added = addUrl(selectedFile);
   }
   else if (fileInfo.isFile())
   {
      if (fileInfo.extension() == "m3u" || fileInfo.extension() == "pls")
         added = addPlaylist(fileInfo);
      else
         added = addFile(fileInfo);
   }
   else if (fileInfo.isDir())
   {
      added = addFolder(fileInfo);
   }

   if (added)
   {
      setupButtons();
      /*
      if (slf->isVisible())
      {
         // restore current selection
         if (kinoPrefs.f_debug)
            qDebug("Restore selected file: - " + saveCurrentFile);

         slf->setCurrentFile(saveCurrentFile);
         slf->setSelected(saveCurrentFile);

         if (kinoPrefs.f_debug)
            qDebug("Current File is: - " + slf->currentFile());

         // move to the next item
         slf->setSelectedNext(true);
      }
      */
   }

   return added;
}


void Player::removeFromPlaylist(void)
{
   if (!delFromPls->isEnabled())
      return;

    QListViewItem *currentItem = contents->playlist->currentItem();
    if (currentItem)
    {
      QString currentFile = currentItem->text(0);
      contents->playlist->removeFromPlaylist();

      // only remove file info if there are no same items in the list
      QListViewItem* item = contents->playlist->firstChild();
      while (item)
      {
         // there is another item with the same name, so don't remove media info
         if (item->text(0) == currentFile)
            return;

         item = item->itemBelow();
      }

      minfo.remove(currentFile);
    }
}


void Player::clearPlaylist(void)
{
   while (contents->playlist->childCount() > 0)
      removeFromPlaylist();

   setupButtons();
}


void Player::loadDefaultPlaylist()
{
   if (kinoPrefs.f_debug)
      qDebug("Player::loadDefaultPlaylist()");

   QFileInfo playlist(QPEApplication::documentDir() + DEFAULT_PLAYLIST);

   // only load default playlist if current playlist is empty
   // and mplayer is not running
   if ((contents->playlist->childCount() == 0) && !process->isRunning() && playlist.exists() && !isAutoplay && !isAddToPlaylist)  // check that we're not already loading
      addPlaylist(playlist);
}

void Player::showPlaylist(bool visible)
{
   contents->setUpdatesEnabled(false);
   if (visible)
   {
      contents->screen->hide();
      contents->albumCover->hide();
      contents->playlist->show();
   }
   else
   {
      contents->playlist->hide();
      contents->screen->show();

      if (m_viewMenu->isItemChecked(VIEW_ALBUM_COVER))
         contents->albumCover->show();
   }

   m_viewMenu->setItemChecked(VIEW_PLAYLIST, visible);
   contents->updateLayout(orientation, m_viewMenu->isItemChecked(VIEW_ALBUM_COVER) && !visible);

   contents->setUpdatesEnabled(true);
   setupButtons();
}

