#include "PlayerIncludes.h"

void Player::loadSettings()
{
    Config cfg("kino2");
    cfg.setGroup("View");
    useScale = cfg.readBoolEntry("Scale");
    m_viewMenu->setItemChecked(VIEW_SCALE, useScale);
//    useFullscreen = cfg.readBoolEntry("Fullscreen");
//    m_viewMenu->setItemChecked(1, useFullscreen);

    bool playlistVisible = cfg.readBoolEntry("playlist", false);
    showPlaylist(playlistVisible);

    bool sliderVisible = cfg.readBoolEntry("slider", true);
    showSlider(sliderVisible);

    bool albumCoverVisible = cfg.readBoolEntry("albumCover", false);
    showAlbumCover(albumCoverVisible);

    cfg.setGroup("Player");

   //display waring message of this is unsupported Zaurus model
   warningDisplayed = cfg.readBoolEntry("warningDisplayed", false);
   if (!warningDisplayed && m_model == UNSUPPORTED)
   {
      QMessageBox::warning(this, tr("Kino 2"), tr("This Zaurus model is unsupported. Application may not run properly!"));
      warningDisplayed = true;
      cfg.writeEntry("warningDisplayed", warningDisplayed);
   }

    kinoPrefs.f_random = cfg.readBoolEntry("random", false);
    kinoPrefs.f_repeat = cfg.readBoolEntry("repeat", false);
    kinoPrefs.f_normal = cfg.readBoolEntry("normal", true);

    kinoPrefs.f_cpu = cfg.readBoolEntry("cpu", false);

    kinoPrefs.s_v_cpu.clock = cfg.readNumEntry("v_cpu_clock", get_freq().clock);
    kinoPrefs.s_a_cpu.clock = cfg.readNumEntry("a_cpu_clock", get_freq().clock);

    kinoPrefs.s_v_cpu.vcore = cfg.readNumEntry("v_cpu_vcore", get_freq().vcore);
    kinoPrefs.s_a_cpu.vcore = cfg.readNumEntry("a_cpu_vcore", get_freq().vcore);

    kinoPrefs.s_priority = cfg.readNumEntry("priority", 0);
    kinoPrefs.f_debug = cfg.readBoolEntry("debug", false);
    kinoPrefs.f_tracking = cfg.readBoolEntry("tracking", true);
    kinoPrefs.f_suspend = cfg.readBoolEntry("suspend", false);
    kinoPrefs.f_lid_suspend = cfg.readBoolEntry("lid_suspend", true);

    kinoPrefs.s_mplayeroptions = cfg.readEntry("mplayeroptions");
    kinoPrefs.s_types = cfg.readEntry("types", DEFAULT_FILE_TYPES);
    kinoPrefs.s_history = cfg.readEntry("history");
    kinoPrefs.s_url_history = cfg.readEntry("url_history");

    cfg.setGroup("Media Stream");
    kinoPrefs.f_cache = cfg.readBoolEntry("cache", false);
    kinoPrefs.s_cache = cfg.readNumEntry("cache_size", 0);
    kinoPrefs.s_bandwidth = cfg.readNumEntry("bandwidth", 0);

    cfg.setGroup("Video");

    // set some default based on current model
    kinoPrefs.f_doublebuffer = cfg.readBoolEntry("doublebuffer", false /*(m_model != SL_C3000)*/);
    kinoPrefs.f_dropframes = cfg.readBoolEntry("framedrop", true);
    kinoPrefs.f_directrendering = cfg.readBoolEntry("directrendering", (m_model == SL_C7X0_860));
    kinoPrefs.f_smart_crop = cfg.readBoolEntry("crop", false);

    // hardware rotate is slow on w100
    kinoPrefs.f_rotate = cfg.readBoolEntry("rotate", false);
    kinoPrefs.f_softscale = cfg.readBoolEntry("softscale", (m_model != SL_C3000 && m_model != SL_C7X0_860));

    kinoPrefs.f_osd_rotate = cfg.readBoolEntry("osd_rotate", true);
    kinoPrefs.f_index = cfg.readBoolEntry("index", false);
    kinoPrefs.f_external_mem = cfg.readBoolEntry("external_mem", (m_model == SL_C7X0_860));
    kinoPrefs.f_noaspect = cfg.readBoolEntry("noaspect", false);
    kinoPrefs.f_osd = cfg.readBoolEntry("osd", false);

    cfg.setGroup("Audio");
    kinoPrefs.f_resample = cfg.readBoolEntry("resample", true);
    kinoPrefs.f_lcd_off = cfg.readBoolEntry("lcd_off", false);
    kinoPrefs.f_sdl = cfg.readBoolEntry("sdl", false);
    kinoPrefs.f_fade = cfg.readBoolEntry("fade", true);
    kinoPrefs.f_swvolume = cfg.readBoolEntry("swvolume", false);
    kinoPrefs.s_maxvolume = cfg.readNumEntry("maxvolume", DEFAULT_MAX_SOFT_VOL);
    kinoPrefs.s_volstep = cfg.readNumEntry("volstep", DEFAULT_VOL_STEP);
    kinoPrefs.f_autosync = cfg.readBoolEntry("autosync", true);
    kinoPrefs.f_hrseek = cfg.readBoolEntry("hrseek", false);

    // if software volume control is enabled, try to read saved setting
    if (kinoPrefs.f_swvolume)
      volumePercent = cfg.readNumEntry("volume", readSystemVolume());
    else
      volumePercent = readSystemVolume();

    cfg.setGroup("Playlist");
    kinoPrefs.f_playonclick = cfg.readBoolEntry("playonclick", false);
    kinoPrefs.f_saveplaylist = cfg.readBoolEntry("saveplaylist", true);

    if (kinoPrefs.f_saveplaylist)
    {
         kinoPrefs.f_playlist_pos = cfg.readBoolEntry("f_playlist_pos", true);
         kinoPrefs.f_file_pos = cfg.readBoolEntry("f_file_pos", false);

         if (kinoPrefs.f_playlist_pos)
            playlist_pos = cfg.readNumEntry("playlist_pos", 0);

         if (kinoPrefs.f_file_pos)
            file_pos = cfg.readNumEntry("file_pos", 0);
    }

    kinoPrefs.f_addontap = cfg.readBoolEntry("addontap", true);

   // load playlist columns
   QHeader* header = contents->playlist->header();
   for (int i = PlaylistView::COLUMN_TRACK; i >= PlaylistView::COLUMN_NAME; i--)
   {
      int pos = cfg.readNumEntry(QString::number(i), i);
      int size = cfg.readNumEntry(QString::number(i) + "w", -1);

      if (i != pos)
         header->moveSection(i, pos);

      if (size < 0)  // first time
         contents->playlist->setColumnWidthMode(i, QListView::Maximum);

//    if (size == 0) // turned off by user
//       contents->playlist->setColumnWidthMode(i, QListView::Manual);

      if (size >= 0)
      {
         contents->playlist->setColumnWidthMode(i, QListView::Manual);
         header->resizeSection(i, size);
      }

      m_columnsMenu->setItemChecked(m_columnsMenu->idAt(i - 2), size != 0);
   }

   contents->playlist->triggerUpdate();

    cfg.setGroup("Controls");
    kinoPrefs.f_remote = cfg.readBoolEntry("remote", true);
    kinoPrefs.f_lid = cfg.readBoolEntry("lid", false);
    kinoPrefs.f_keyboard = cfg.readBoolEntry("keyboard", true);

    kinoPrefs.keyboardActions[ACTION_PLAY_STOP] = cfg.readNumEntry("PlayStop_K", Key_Space);
    kinoPrefs.keyboardActions[ACTION_PAUSE_RESUME] = cfg.readNumEntry("PauseResume_K", Key_Return);
    kinoPrefs.keyboardActions[ACTION_VOLUME_UP] = cfg.readNumEntry("VolumeUp_K", Key_Up);
    kinoPrefs.keyboardActions[ACTION_VOLUME_DOWN] = cfg.readNumEntry("VolumeDown_K", Key_Down);
    kinoPrefs.keyboardActions[ACTION_MUTE] = cfg.readNumEntry("Mute_K", Key_M);
    kinoPrefs.keyboardActions[ACTION_SEEK_F10] = cfg.readNumEntry("Forward10_K", Key_Right);
    kinoPrefs.keyboardActions[ACTION_SEEK_B10] = cfg.readNumEntry("Backward10_K", Key_Left);
    kinoPrefs.keyboardActions[ACTION_SEEK_F60] = cfg.readNumEntry("Forward60_K", Key_C);
    kinoPrefs.keyboardActions[ACTION_SEEK_B60] = cfg.readNumEntry("Backward60_K", Key_X);
    kinoPrefs.keyboardActions[ACTION_SEEK_F600] = cfg.readNumEntry("Forward600_K", Key_V);
    kinoPrefs.keyboardActions[ACTION_SEEK_B600] = cfg.readNumEntry("Backward600_K", Key_Z);
    kinoPrefs.keyboardActions[ACTION_NEXT] = cfg.readNumEntry("Next_K", Key_N); // Key_Down + RemoteControlButton::KEY_DOUBLE;
    kinoPrefs.keyboardActions[ACTION_PREV] = cfg.readNumEntry("Prev_K", Key_B); // Key_Up + RemoteControlButton::KEY_DOUBLE;
    kinoPrefs.keyboardActions[ACTION_OSD] = cfg.readNumEntry("OSD_K", Key_O);
    kinoPrefs.keyboardActions[ACTION_SUBTITLES] = cfg.readNumEntry("Subtitles_K", Key_S);
    kinoPrefs.keyboardActions[ACTION_LCD] = cfg.readNumEntry("LCD_K", Key_Q);

    kinoPrefs.keyboardActions[ACTION_ADD_TO_PLAYLIST] = cfg.readNumEntry("AddToPlaylist_K", Key_A);
    kinoPrefs.keyboardActions[ACTION_ADD_ALL_TO_PLAYLIST] = cfg.readNumEntry("AddAllToPlaylist_K", Key_W);
    kinoPrefs.keyboardActions[ACTION_REMOVE_FROM_PLAYLIST] = cfg.readNumEntry("RemoveFromPlaylist_K", Key_R);
    kinoPrefs.keyboardActions[ACTION_CLEAR_PLAYLIST] = cfg.readNumEntry("ClearPlaylist_K", Key_L);
    kinoPrefs.keyboardActions[ACTION_PLAYLIST_UP] = cfg.readNumEntry("PlaylistUp_K", Key_U);
    kinoPrefs.keyboardActions[ACTION_PLAYLIST_DOWN] = cfg.readNumEntry("PlaylistDown_K", Key_D);
    kinoPrefs.keyboardActions[ACTION_FILE_INFO] = cfg.readNumEntry("FileInfo_K", Key_I);

    kinoPrefs.remoteActions[ACTION_PLAY_STOP] = cfg.readNumEntry("PlayStop_R", RemoteControlButton::KEY_STOP);
    kinoPrefs.remoteActions[ACTION_PAUSE_RESUME] = cfg.readNumEntry("PauseResume_R", RemoteControlButton::KEY_PLAY_PAUSE);
    kinoPrefs.remoteActions[ACTION_VOLUME_UP] = cfg.readNumEntry("VolumeUp_R", RemoteControlButton::KEY_VOLUME_UP);
    kinoPrefs.remoteActions[ACTION_VOLUME_DOWN] = cfg.readNumEntry("VolumeDown_R", RemoteControlButton::KEY_VOLUME_DOWN);
    kinoPrefs.remoteActions[ACTION_MUTE] = cfg.readNumEntry("Mute_R", RemoteControlButton::KEY_VOLUME_DOWN + RemoteControlButton::KEY_DOUBLE);
    kinoPrefs.remoteActions[ACTION_SEEK_F10] = cfg.readNumEntry("Forward10_R", RemoteControlButton::KEY_FORWARD);
    kinoPrefs.remoteActions[ACTION_SEEK_B10] = cfg.readNumEntry("Backward10_R", RemoteControlButton::KEY_BACKWARD);
    kinoPrefs.remoteActions[ACTION_SEEK_F60] = cfg.readNumEntry("Forward60_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_SEEK_B60] = cfg.readNumEntry("Backward60_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_SEEK_F600] = cfg.readNumEntry("Forward600_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_SEEK_B600] = cfg.readNumEntry("Backward600_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_NEXT] = cfg.readNumEntry("Next_R", RemoteControlButton::KEY_FORWARD + RemoteControlButton::KEY_DOUBLE);
    kinoPrefs.remoteActions[ACTION_PREV] = cfg.readNumEntry("Prev_R", RemoteControlButton::KEY_BACKWARD + RemoteControlButton::KEY_DOUBLE);
    kinoPrefs.remoteActions[ACTION_OSD] = cfg.readNumEntry("OSD_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_LCD] = cfg.readNumEntry("LCD_R", RemoteControlButton::KEY_PLAY_PAUSE + RemoteControlButton::KEY_DOUBLE);

    kinoPrefs.remoteActions[ACTION_SUBTITLES] = cfg.readNumEntry("Subtitles_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_ADD_TO_PLAYLIST] = cfg.readNumEntry("AddToPlaylist_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_ADD_ALL_TO_PLAYLIST] = cfg.readNumEntry("AddAllToPlaylist_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_REMOVE_FROM_PLAYLIST] = cfg.readNumEntry("RemoveFromPlaylist_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_CLEAR_PLAYLIST] = cfg.readNumEntry("ClearPlaylist_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_PLAYLIST_UP] = cfg.readNumEntry("PlaylistUp_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_PLAYLIST_DOWN] = cfg.readNumEntry("PlaylistDown_R", RemoteControlButton::KEY_NONE);
    kinoPrefs.remoteActions[ACTION_FILE_INFO] = cfg.readNumEntry("FileInfo_R", RemoteControlButton::KEY_STOP + RemoteControlButton::KEY_DOUBLE);

    // Encodings
    kinoPrefs.s_enc_mp3 = cfg.readEntry("enc_mp3", QTextCodec::codecForLocale()->name());
    kinoPrefs.s_enc_playlist = cfg.readEntry("enc_playlist", QTextCodec::codecForLocale()->name());
    kinoPrefs.s_font = cfg.readEntry("font", "helvetica");
    kinoPrefs.s_fontSize = cfg.readNumEntry("fontSize", 24);

   // read user display settings related
    Config qpeconfig("qpe");
    if (qpeconfig.isValid())
    {
      //qpeconfig->setGroup("BatteryPower");
      qpeconfig.setGroup("Screensaver");
      saved_lid_suspend = qpeconfig.readBoolEntry("CoverSuspend", true);

      if (qpeconfig.hasKey("Interval"))
      {
         intervalSuspend = qpeconfig.readNumEntry("Interval", 60);
         intervalLightOff = qpeconfig.readNumEntry("interval_LightOff", 0);
         dim = qpeconfig.readNumEntry("Dim", 1);
         intervalDim = qpeconfig.readNumEntry("Interval_Dim", 30);
      }
   }

}

void Player::saveSettings()
{
    Config cfg("kino2");
    cfg.setGroup("View");
    cfg.writeEntry("Scale", useScale);
    cfg.writeEntry("playlist", m_viewMenu->isItemChecked(VIEW_PLAYLIST));
    cfg.writeEntry("slider", m_viewMenu->isItemChecked(VIEW_SLIDER));
    cfg.writeEntry("albumCover", m_viewMenu->isItemChecked(VIEW_ALBUM_COVER));

    cfg.setGroup("Player");
    cfg.writeEntry("random", kinoPrefs.f_random);
    cfg.writeEntry("repeat", kinoPrefs.f_repeat);
    cfg.writeEntry("normal", kinoPrefs.f_normal);

    cfg.writeEntry("cpu", kinoPrefs.f_cpu);

    cfg.writeEntry("v_cpu_clock", kinoPrefs.s_v_cpu.clock);
    cfg.writeEntry("a_cpu_clock", kinoPrefs.s_a_cpu.clock);
    cfg.writeEntry("v_cpu_vcore", kinoPrefs.s_v_cpu.vcore);
    cfg.writeEntry("a_cpu_vcore", kinoPrefs.s_a_cpu.vcore);

    cfg.writeEntry("priority", kinoPrefs.s_priority);
    cfg.writeEntry("debug", kinoPrefs.f_debug);
    cfg.writeEntry("tracking", kinoPrefs.f_tracking);
    cfg.writeEntry("suspend", kinoPrefs.f_suspend);
    cfg.writeEntry("lid_suspend", kinoPrefs.f_lid_suspend);

    cfg.writeEntry("mplayeroptions", kinoPrefs.s_mplayeroptions);
    cfg.writeEntry("types", kinoPrefs.s_types);

//    kinoPrefs.s_history = "";
//    QStringList list = slf->recent();
//    for (QStringList::Iterator it = list.begin(); it != list.end(); ++it )
//    {
//      kinoPrefs.s_history += (*it).utf8();
//      kinoPrefs.s_history += ",";
//    }

    cfg.writeEntry("history", kinoPrefs.s_history);
    cfg.writeEntry("url_history", kinoPrefs.s_url_history);

    cfg.setGroup("Media Stream");
    cfg.writeEntry("cache", kinoPrefs.f_cache);
    cfg.writeEntry("cache_size", kinoPrefs.s_cache);
    cfg.writeEntry("bandwidth", kinoPrefs.s_bandwidth);

    cfg.setGroup("Video");
    cfg.writeEntry("doublebuffer", kinoPrefs.f_doublebuffer);
    cfg.writeEntry("framedrop", kinoPrefs.f_dropframes);
    cfg.writeEntry("directrendering", kinoPrefs.f_directrendering);
    cfg.writeEntry("crop", kinoPrefs.f_smart_crop);
    cfg.writeEntry("rotate", kinoPrefs.f_rotate);
    cfg.writeEntry("softscale", kinoPrefs.f_softscale);
    cfg.writeEntry("osd_rotate", kinoPrefs.f_osd_rotate);
    cfg.writeEntry("index", kinoPrefs.f_index);
    cfg.writeEntry("external_mem", kinoPrefs.f_external_mem);
    cfg.writeEntry("noaspect", kinoPrefs.f_noaspect);
    cfg.writeEntry("osd", kinoPrefs.f_osd);

    cfg.setGroup("Audio");
    cfg.writeEntry("resample", kinoPrefs.f_resample);
    cfg.writeEntry("lcd_off", kinoPrefs.f_lcd_off);
    cfg.writeEntry("sdl", kinoPrefs.f_sdl);
    cfg.writeEntry("fade", kinoPrefs.f_fade);
    cfg.writeEntry("swvolume", kinoPrefs.f_swvolume);
    cfg.writeEntry("maxvolume", kinoPrefs.s_maxvolume);
    cfg.writeEntry("volstep", kinoPrefs.s_volstep);
    cfg.writeEntry("volume", volumePercent);
    cfg.writeEntry("autosync", kinoPrefs.f_autosync);
    cfg.writeEntry("hrseek", kinoPrefs.f_hrseek);

    cfg.setGroup("Playlist");
    cfg.writeEntry("playonclick", kinoPrefs.f_playonclick);
    cfg.writeEntry("addontap", kinoPrefs.f_addontap);
    cfg.writeEntry("saveplaylist", kinoPrefs.f_saveplaylist);

    if (kinoPrefs.f_saveplaylist)
    {
       playlist_pos = contents->playlist->currentItemIndex();
       file_pos = currentPosition;

       cfg.writeEntry("f_playlist_pos", kinoPrefs.f_playlist_pos);
       cfg.writeEntry("playlist_pos", playlist_pos);

       cfg.writeEntry("f_file_pos", kinoPrefs.f_file_pos);
       cfg.writeEntry("file_pos", file_pos);
    }

   // save playlist columns
   QHeader* header = contents->playlist->header();
   for (int i = PlaylistView::COLUMN_NAME; i <= PlaylistView::COLUMN_TRACK; i++)
   {
      cfg.writeEntry(QString::number(i), header->mapToIndex(i));
      cfg.writeEntry(QString::number(i) + "w", header->sectionSize(i));
   }

    cfg.setGroup("Controls");
    cfg.writeEntry("remote", kinoPrefs.f_remote);
    {
       Config cfg("qpe");
       cfg.setGroup("Remocon");
       cfg.writeEntry("master", kinoPrefs.f_remote ? "kino2" : "musicplayer");
    }

   cfg.writeEntry("lid", kinoPrefs.f_lid);
   cfg.writeEntry("keyboard", kinoPrefs.f_keyboard);

    cfg.writeEntry("PlayStop_K", kinoPrefs.keyboardActions[ACTION_PLAY_STOP]);
    cfg.writeEntry("PauseResume_K", kinoPrefs.keyboardActions[ACTION_PAUSE_RESUME]);
    cfg.writeEntry("VolumeUp_K", kinoPrefs.keyboardActions[ACTION_VOLUME_UP]);
    cfg.writeEntry("VolumeDown_K", kinoPrefs.keyboardActions[ACTION_VOLUME_DOWN]);
    cfg.writeEntry("Mute_K", kinoPrefs.keyboardActions[ACTION_MUTE]);
    cfg.writeEntry("Forward10_K", kinoPrefs.keyboardActions[ACTION_SEEK_F10]);
    cfg.writeEntry("Backward10_K", kinoPrefs.keyboardActions[ACTION_SEEK_B10]);
    cfg.writeEntry("Forward60_K", kinoPrefs.keyboardActions[ACTION_SEEK_F60]);
    cfg.writeEntry("Backward60_K", kinoPrefs.keyboardActions[ACTION_SEEK_B60]);
    cfg.writeEntry("Forward600_K", kinoPrefs.keyboardActions[ACTION_SEEK_F600]);
    cfg.writeEntry("Backward600_K", kinoPrefs.keyboardActions[ACTION_SEEK_B600]);
    cfg.writeEntry("Next_K", kinoPrefs.keyboardActions[ACTION_NEXT]);
    cfg.writeEntry("Prev_K", kinoPrefs.keyboardActions[ACTION_PREV]);
    cfg.writeEntry("OSD_K", kinoPrefs.keyboardActions[ACTION_OSD]);
    cfg.writeEntry("Subtitles_K", kinoPrefs.keyboardActions[ACTION_SUBTITLES]);
    cfg.writeEntry("LCD_K", kinoPrefs.keyboardActions[ACTION_LCD]);

    cfg.writeEntry("AddToPlaylist_K", kinoPrefs.keyboardActions[ACTION_ADD_TO_PLAYLIST]);
    cfg.writeEntry("AddAllToPlaylist_K", kinoPrefs.keyboardActions[ACTION_ADD_ALL_TO_PLAYLIST]);
    cfg.writeEntry("RemoveFromPlaylist_K", kinoPrefs.keyboardActions[ACTION_REMOVE_FROM_PLAYLIST]);
    cfg.writeEntry("ClearPlaylist_K", kinoPrefs.keyboardActions[ACTION_CLEAR_PLAYLIST]);
    cfg.writeEntry("PlaylistUp_K", kinoPrefs.keyboardActions[ACTION_PLAYLIST_UP]);
    cfg.writeEntry("PlaylistDown_K", kinoPrefs.keyboardActions[ACTION_PLAYLIST_DOWN]);
    cfg.writeEntry("FileInfo_K", kinoPrefs.keyboardActions[ACTION_FILE_INFO]);

    cfg.writeEntry("PlayStop_R", kinoPrefs.remoteActions[ACTION_PLAY_STOP]);
    cfg.writeEntry("PauseResume_R", kinoPrefs.remoteActions[ACTION_PAUSE_RESUME]);
    cfg.writeEntry("VolumeUp_R", kinoPrefs.remoteActions[ACTION_VOLUME_UP]);
    cfg.writeEntry("VolumeDown_R", kinoPrefs.remoteActions[ACTION_VOLUME_DOWN]);
    cfg.writeEntry("Mute_R", kinoPrefs.remoteActions[ACTION_MUTE]);
    cfg.writeEntry("Forward10_R", kinoPrefs.remoteActions[ACTION_SEEK_F10]);
    cfg.writeEntry("Backward10_R", kinoPrefs.remoteActions[ACTION_SEEK_B10]);
    cfg.writeEntry("Forward60_R", kinoPrefs.remoteActions[ACTION_SEEK_F60]);
    cfg.writeEntry("Backward60_R", kinoPrefs.remoteActions[ACTION_SEEK_B60]);
    cfg.writeEntry("Forward600_R", kinoPrefs.remoteActions[ACTION_SEEK_F600]);
    cfg.writeEntry("Backward600_R", kinoPrefs.remoteActions[ACTION_SEEK_B600]);
    cfg.writeEntry("Next_R", kinoPrefs.remoteActions[ACTION_NEXT]);
    cfg.writeEntry("Prev_R", kinoPrefs.remoteActions[ACTION_PREV]);
    cfg.writeEntry("OSD_R", kinoPrefs.remoteActions[ACTION_OSD]);
    cfg.writeEntry("Subtitles_R", kinoPrefs.remoteActions[ACTION_SUBTITLES]);
    cfg.writeEntry("LCD_R", kinoPrefs.remoteActions[ACTION_LCD]);

    cfg.writeEntry("AddToPlaylist_R", kinoPrefs.remoteActions[ACTION_ADD_TO_PLAYLIST]);
    cfg.writeEntry("AddAllToPlaylist_R", kinoPrefs.remoteActions[ACTION_ADD_ALL_TO_PLAYLIST]);
    cfg.writeEntry("RemoveFromPlaylist_R", kinoPrefs.remoteActions[ACTION_REMOVE_FROM_PLAYLIST]);
    cfg.writeEntry("ClearPlaylist_R", kinoPrefs.remoteActions[ACTION_CLEAR_PLAYLIST]);
    cfg.writeEntry("PlaylistUp_R", kinoPrefs.remoteActions[ACTION_PLAYLIST_UP]);
    cfg.writeEntry("PlaylistDown_R", kinoPrefs.remoteActions[ACTION_PLAYLIST_DOWN]);
    cfg.writeEntry("FileInfo_R", kinoPrefs.remoteActions[ACTION_FILE_INFO]);

    // Encodings
    cfg.writeEntry("enc_mp3", kinoPrefs.s_enc_mp3);
    cfg.writeEntry("enc_playlist", kinoPrefs.s_enc_playlist);
    cfg.writeEntry("font", kinoPrefs.s_font);
    cfg.writeEntry("fontSize", kinoPrefs.s_fontSize);
}
