#include <qmessagebox.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qcstring.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qscrollview.h>
#include <qgroupbox.h>
#include <qspinbox.h>

#include "kinosettings.h"
#include "player.h"

#include "SelectKeyButton.h"
#include "RemoteControlButton.h"
#include "FrequenciesCombo.h"
#include "EncodingsCombo.h"
#include "FontsCombo.h"

KinoSettings::KinoSettings( QWidget* parent,  const QString &name, bool modal, WFlags fl )
    : KinoSettingsBase( parent, name, modal, fl )
{
	// move controls into the scroll view
	QScrollView *sv = new QScrollView(groupMappings);
	groupMappingsLayout->addWidget(sv, 0, 0);

	sv->setResizePolicy(QScrollView::AutoOneFit);
	sv->setFrameStyle(QFrame::NoFrame);
	sv->setVScrollBarMode(QScrollView::Auto);
	sv->setHScrollBarMode(QScrollView::Auto);
	sv->addChild(frameMappings);

//	f_fade->hide();
	f_osd_rotate->hide();
   f_external_mem->hide();
   f_sdl->hide();
   f_directrendering->hide();
//   f_softscale->hide();

   s_maxvolume->setEnabled(f_swvolume->isChecked());

   f_playlist_pos->setEnabled(f_saveplaylist->isChecked());
   f_file_pos->setEnabled(f_saveplaylist->isChecked());

   setCaption(name);
   showMaximized();
}

KinoSettings::~KinoSettings()
{
}

void KinoSettings::setSettings(KinoPrefs p)
{
	// General
	f_normal->setChecked(p.f_normal);
	f_repeat->setChecked(p.f_repeat);
	f_random->setChecked(p.f_random);

	f_cpu->setChecked(p.f_cpu);
	s_a_cpu->setEnabled(p.f_cpu);
	s_v_cpu->setEnabled(p.f_cpu);
   f_suspend->setChecked(p.f_suspend);
   f_lid_suspend->setChecked(p.f_lid_suspend);

	s_a_cpu->setCurrentFrequency(p.s_a_cpu);
	s_v_cpu->setCurrentFrequency(p.s_v_cpu);

	s_priority->setText(QString::number(p.s_priority));

	// Media Stream
	f_cache->setChecked(p.f_cache);
	s_cache->setEnabled(p.f_cache);
    s_cache->setText(QString::number(p.s_cache));
    s_bandwidth->setText(QString::number(p.s_bandwidth));

	s_mplayeroptions->setText(p.s_mplayeroptions);
	f_debug->setChecked(p.f_debug);
   s_types->setText(p.s_types);

    // Video
    f_dropframes->setChecked(p.f_dropframes);
    f_directrendering->setChecked(p.f_directrendering);
    f_doublebuffer->setChecked(p.f_doublebuffer);
    f_smart_crop->setChecked(p.f_smart_crop);
    f_rotate->setChecked(p.f_rotate);
    f_softscale->setChecked(p.f_softscale);
    f_osd_rotate->setChecked(p.f_osd_rotate);
    f_index->setChecked(p.f_index);
    f_external_mem->setChecked(p.f_external_mem);
    f_noaspect->setChecked(p.f_noaspect);
    f_osd->setChecked(p.f_osd);

    // Audio
    f_resample->setChecked(p.f_resample);
	 f_lcd_off->setChecked(p.f_lcd_off);
	 f_sdl->setChecked(p.f_sdl);
	 f_fade->setChecked(p.f_fade);
    f_swvolume->setChecked(p.f_swvolume);
    s_maxvolume->setText(QString::number(p.s_maxvolume));
    s_volstep->setText(QString::number(p.s_volstep));
    f_tracking->setChecked(p.f_tracking);
    f_autosync->setChecked(p.f_autosync);
    f_hrseek->setChecked(p.f_hrseek);

   // Playlist
   f_playonclick->setChecked(p.f_playonclick);
   f_saveplaylist->setChecked(p.f_saveplaylist);
   f_playlist_pos->setChecked(p.f_playlist_pos);
   f_file_pos->setChecked(p.f_file_pos);
   f_addontap->setChecked(p.f_addontap);
   s_enc_mp3->setCurrentCodec(p.s_enc_mp3);
   s_enc_playlist->setCurrentCodec(p.s_enc_playlist);
   s_font->setCurrentFont(p.s_font);
   s_fontSize->setValue(p.s_fontSize);

	// Controls
	f_remote->setChecked(p.f_remote);
	f_lid->setChecked(p.f_lid);
   f_keyboard->setChecked(p.f_keyboard);

    cbPlayStop_K->setCurrentKey(p.keyboardActions[Player::ACTION_PLAY_STOP]);
    cbPauseResume_K->setCurrentKey(p.keyboardActions[Player::ACTION_PAUSE_RESUME]);
    cbVolumeUp_K->setCurrentKey(p.keyboardActions[Player::ACTION_VOLUME_UP]);
    cbVolumeDown_K->setCurrentKey(p.keyboardActions[Player::ACTION_VOLUME_DOWN]);
    cbMute_K->setCurrentKey(p.keyboardActions[Player::ACTION_MUTE]);
    cbForward10_K->setCurrentKey(p.keyboardActions[Player::ACTION_SEEK_F10]);
    cbBackward10_K->setCurrentKey(p.keyboardActions[Player::ACTION_SEEK_B10]);
    cbForward60_K->setCurrentKey(p.keyboardActions[Player::ACTION_SEEK_F60]);
    cbBackward60_K->setCurrentKey(p.keyboardActions[Player::ACTION_SEEK_B60]);
    cbForward600_K->setCurrentKey(p.keyboardActions[Player::ACTION_SEEK_F600]);
    cbBackward600_K->setCurrentKey(p.keyboardActions[Player::ACTION_SEEK_B600]);
    cbNext_K->setCurrentKey(p.keyboardActions[Player::ACTION_NEXT]);
    cbPrev_K->setCurrentKey(p.keyboardActions[Player::ACTION_PREV]);
    cbOSD_K->setCurrentKey(p.keyboardActions[Player::ACTION_OSD]);
    cbSubtitles_K->setCurrentKey(p.keyboardActions[Player::ACTION_SUBTITLES]);
    cbLCD_K->setCurrentKey(p.keyboardActions[Player::ACTION_LCD]);

    cbAddToPlaylist_K->setCurrentKey(p.keyboardActions[Player::ACTION_ADD_TO_PLAYLIST]);
    cbAddAllToPlaylist_K->setCurrentKey(p.keyboardActions[Player::ACTION_ADD_ALL_TO_PLAYLIST]);
    cbRemoveFromPlaylist_K->setCurrentKey(p.keyboardActions[Player::ACTION_REMOVE_FROM_PLAYLIST]);
    cbClearPlaylist_K->setCurrentKey(p.keyboardActions[Player::ACTION_CLEAR_PLAYLIST]);
    cbPlaylistUp_K->setCurrentKey(p.keyboardActions[Player::ACTION_PLAYLIST_UP]);
    cbPlaylistDown_K->setCurrentKey(p.keyboardActions[Player::ACTION_PLAYLIST_DOWN]);
    cbFileInfo_K->setCurrentKey(p.keyboardActions[Player::ACTION_FILE_INFO]);

    cbPlayStop_R->setCurrentKey(p.remoteActions[Player::ACTION_PLAY_STOP]);
    cbPauseResume_R->setCurrentKey(p.remoteActions[Player::ACTION_PAUSE_RESUME]);
    cbVolumeUp_R->setCurrentKey(p.remoteActions[Player::ACTION_VOLUME_UP]);
    cbVolumeDown_R->setCurrentKey(p.remoteActions[Player::ACTION_VOLUME_DOWN]);
    cbMute_R->setCurrentKey(p.remoteActions[Player::ACTION_MUTE]);
    cbForward10_R->setCurrentKey(p.remoteActions[Player::ACTION_SEEK_F10]);
    cbBackward10_R->setCurrentKey(p.remoteActions[Player::ACTION_SEEK_B10]);
    cbForward60_R->setCurrentKey(p.remoteActions[Player::ACTION_SEEK_F60]);
    cbBackward60_R->setCurrentKey(p.remoteActions[Player::ACTION_SEEK_B60]);
    cbForward600_R->setCurrentKey(p.remoteActions[Player::ACTION_SEEK_F600]);
    cbBackward600_R->setCurrentKey(p.remoteActions[Player::ACTION_SEEK_B600]);
    cbNext_R->setCurrentKey(p.remoteActions[Player::ACTION_NEXT]);
    cbPrev_R->setCurrentKey(p.remoteActions[Player::ACTION_PREV]);
    cbOSD_R->setCurrentKey(p.remoteActions[Player::ACTION_OSD]);
    cbSubtitles_R->setCurrentKey(p.remoteActions[Player::ACTION_SUBTITLES]);
    cbLCD_R->setCurrentKey(p.remoteActions[Player::ACTION_LCD]);

    cbAddToPlaylist_R->setCurrentKey(p.remoteActions[Player::ACTION_ADD_TO_PLAYLIST]);
    cbAddAllToPlaylist_R->setCurrentKey(p.remoteActions[Player::ACTION_ADD_ALL_TO_PLAYLIST]);
    cbRemoveFromPlaylist_R->setCurrentKey(p.remoteActions[Player::ACTION_REMOVE_FROM_PLAYLIST]);
    cbClearPlaylist_R->setCurrentKey(p.remoteActions[Player::ACTION_CLEAR_PLAYLIST]);
    cbPlaylistUp_R->setCurrentKey(p.remoteActions[Player::ACTION_PLAYLIST_UP]);
    cbPlaylistDown_R->setCurrentKey(p.remoteActions[Player::ACTION_PLAYLIST_DOWN]);
    cbFileInfo_R->setCurrentKey(p.remoteActions[Player::ACTION_FILE_INFO]);
}

KinoPrefs KinoSettings::getSettings(void)
{
    KinoPrefs p;

    // General
    p.f_normal = f_normal->isChecked();
    p.f_repeat = f_repeat->isChecked();
    p.f_random = f_random->isChecked();

    p.f_cpu = f_cpu->isChecked();
	 p.s_v_cpu = s_v_cpu->currentFrequency();
	 p.s_a_cpu = s_a_cpu->currentFrequency();
	 p.s_priority = s_priority->text().toInt(0, 10);
    p.f_suspend = f_suspend->isChecked();
    p.f_lid_suspend = f_lid_suspend->isChecked();

    // Media Stream
	 p.f_cache = f_cache->isChecked();
    p.s_cache = s_cache->text().toInt(0, 10);
    p.s_bandwidth = s_bandwidth->text().toInt(0, 10);

	 p.s_mplayeroptions = s_mplayeroptions->text();
    p.f_debug = f_debug->isChecked();

	 p.s_types = s_types->text();

    // Video
    p.f_dropframes = f_dropframes->isChecked();
    p.f_directrendering = f_directrendering->isChecked();
    p.f_doublebuffer = f_doublebuffer->isChecked();
    p.f_smart_crop = f_smart_crop->isChecked();
    p.f_rotate = f_rotate->isChecked();
    p.f_softscale = f_softscale->isChecked();
    p.f_osd_rotate = f_osd_rotate->isChecked();
    p.f_index = f_index->isChecked();
    p.f_external_mem = f_external_mem->isChecked();
    p.f_noaspect = f_noaspect->isChecked();
    p.f_osd = f_osd->isChecked();

	// Audio
    p.f_resample = f_resample->isChecked();
    p.f_lcd_off = f_lcd_off->isChecked();
    p.f_sdl = f_sdl->isChecked();
    p.f_fade = f_fade->isChecked();
    p.f_swvolume = f_swvolume->isChecked();
    p.s_maxvolume = s_maxvolume->text().toInt(0, 10);
    p.s_volstep = s_volstep->text().toInt(0, 10);
    p.f_tracking = f_tracking->isChecked();
    p.f_autosync = f_autosync->isChecked();
    p.f_hrseek = f_hrseek->isChecked();

    // Playlist
    p.f_playonclick = f_playonclick->isChecked();
    p.f_saveplaylist = f_saveplaylist->isChecked();
    p.f_playlist_pos = f_playlist_pos->isChecked();
    p.f_file_pos = f_file_pos->isChecked();
    p.f_addontap = f_addontap->isChecked();

    p.s_enc_mp3 = s_enc_mp3->currentCodec();
    p.s_enc_playlist = s_enc_playlist->currentCodec();
    p.s_font = s_font->currentFont();
    p.s_fontSize = s_fontSize->value();

    // Controls
    p.f_remote = f_remote->isChecked();
    p.f_lid = f_lid->isChecked();
    p.f_keyboard = f_keyboard->isChecked();

    p.keyboardActions[Player::ACTION_PLAY_STOP] = cbPlayStop_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_PAUSE_RESUME] = cbPauseResume_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_VOLUME_UP] = cbVolumeUp_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_VOLUME_DOWN] = cbVolumeDown_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_MUTE] = cbMute_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_SEEK_F10] = cbForward10_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_SEEK_B10] = cbBackward10_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_SEEK_F60] = cbForward60_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_SEEK_B60] = cbBackward60_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_SEEK_F600] = cbForward600_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_SEEK_B600] = cbBackward600_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_NEXT] = cbNext_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_PREV] = cbPrev_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_OSD] = cbOSD_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_SUBTITLES] = cbSubtitles_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_LCD] = cbLCD_K->getCurrentKey();

    p.keyboardActions[Player::ACTION_ADD_TO_PLAYLIST] = cbAddToPlaylist_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_ADD_ALL_TO_PLAYLIST] = cbAddAllToPlaylist_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_REMOVE_FROM_PLAYLIST] = cbRemoveFromPlaylist_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_CLEAR_PLAYLIST] = cbClearPlaylist_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_PLAYLIST_UP] = cbPlaylistUp_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_PLAYLIST_DOWN] = cbPlaylistDown_K->getCurrentKey();
    p.keyboardActions[Player::ACTION_FILE_INFO] = cbFileInfo_K->getCurrentKey();

    p.remoteActions[Player::ACTION_PLAY_STOP] = cbPlayStop_R->getCurrentKey();
    p.remoteActions[Player::ACTION_PAUSE_RESUME] = cbPauseResume_R->getCurrentKey();
    p.remoteActions[Player::ACTION_VOLUME_UP] = cbVolumeUp_R->getCurrentKey();
    p.remoteActions[Player::ACTION_VOLUME_DOWN] = cbVolumeDown_R->getCurrentKey();
    p.remoteActions[Player::ACTION_MUTE] = cbMute_R->getCurrentKey();
    p.remoteActions[Player::ACTION_SEEK_F10] = cbForward10_R->getCurrentKey();
    p.remoteActions[Player::ACTION_SEEK_B10] = cbBackward10_R->getCurrentKey();
    p.remoteActions[Player::ACTION_SEEK_F60] = cbForward60_R->getCurrentKey();
    p.remoteActions[Player::ACTION_SEEK_B60] = cbBackward60_R->getCurrentKey();
    p.remoteActions[Player::ACTION_SEEK_F600] = cbForward600_R->getCurrentKey();
    p.remoteActions[Player::ACTION_SEEK_B600] = cbBackward600_R->getCurrentKey();
    p.remoteActions[Player::ACTION_NEXT] = cbNext_R->getCurrentKey();
    p.remoteActions[Player::ACTION_PREV] = cbPrev_R->getCurrentKey();
    p.remoteActions[Player::ACTION_OSD] = cbOSD_R->getCurrentKey();
    p.remoteActions[Player::ACTION_SUBTITLES] = cbSubtitles_R->getCurrentKey();
    p.remoteActions[Player::ACTION_LCD] = cbLCD_R->getCurrentKey();

    p.remoteActions[Player::ACTION_ADD_TO_PLAYLIST] = cbAddToPlaylist_R->getCurrentKey();
    p.remoteActions[Player::ACTION_ADD_ALL_TO_PLAYLIST] = cbAddAllToPlaylist_R->getCurrentKey();
    p.remoteActions[Player::ACTION_REMOVE_FROM_PLAYLIST] = cbRemoveFromPlaylist_R->getCurrentKey();
    p.remoteActions[Player::ACTION_CLEAR_PLAYLIST] = cbClearPlaylist_R->getCurrentKey();
    p.remoteActions[Player::ACTION_PLAYLIST_UP] = cbPlaylistUp_R->getCurrentKey();
    p.remoteActions[Player::ACTION_PLAYLIST_DOWN] = cbPlaylistDown_R->getCurrentKey();
    p.remoteActions[Player::ACTION_FILE_INFO] = cbFileInfo_R->getCurrentKey();

    return p;
}

void KinoSettings::cacheToggled(bool enable)
{
	s_cache->setEnabled(enable);
}

void KinoSettings::cpuToggled(bool enable)
{
	s_v_cpu->setEnabled(enable);
	s_a_cpu->setEnabled(enable);
}
