/********************************
 * (c) Cacko Team, 2002 - 2005
 * Kino2 Media Player
 * http://cacko.biz/kino2/
 *******************************/

#include "PlayerIncludes.h"

void PlayerBaseImpl::updateLayout(Qt::Orientation orientation, bool albumVisible)
{
   setUpdatesEnabled(false);
   if (orientation == Vertical)
   {
      qDebug("PlayerBaseImpl::updateLayout() - Vertical");

      PlayerBaseLayout->addMultiCellWidget( albumCover, 0, 0, 0, 1 );

      if (albumVisible)
      {
         PlayerBaseLayout->addMultiCellWidget( playlist, 1, 1, 0, 1 );
         PlayerBaseLayout->addMultiCellWidget( screen, 1, 1, 0, 1 );
      }
      else
      {
         PlayerBaseLayout->addMultiCellWidget( playlist, 0, 1, 0, 1 );
         PlayerBaseLayout->addMultiCellWidget( screen, 0, 1, 0, 1 );
      }
   }
   else  // default
   {
      qDebug("PlayerBaseImpl::updateLayout() - Horizontal");

      PlayerBaseLayout->addMultiCellWidget( albumCover, 0, 1, 0, 0 );
      PlayerBaseLayout->addMultiCellWidget( playlist, 0, 1, 1, 1 );
      PlayerBaseLayout->addMultiCellWidget( screen, 0, 1, 1, 1 );
   }

   progress->hide();
   time->hide();

   progress->show();
   time->show();

   setUpdatesEnabled(true);
   PlayerBaseLayout->invalidate();
   updateGeometry();
   update();
}


Player::Player( QWidget* parent, const char* name, WFlags fl )
    : QMainWindow( parent, name, fl ), isPlaying(false), isPausing(false), isIdentify(false),
    	isAddToPlaylist(false), isLCDBlank(false), isAutoplay(false), isVideoFile(false), isIgnoreOutput(false),
    	isSelectFirst(false), doubleKey(false), needToRefresh(false), positionRestored(false),
      m_model(UNSUPPORTED), playlist_pos(0), file_pos(0)
{
    srand((unsigned)time(0));

	 setSelectedFile("");
    setToolBarsMovable(false);
    setFocusPolicy(QWidget::StrongFocus);

    m_fileMenu = new QPopupMenu(this);
    connect(m_fileMenu, SIGNAL(activated(int)), this, SLOT(fileMenu(int)));
    m_fileMenu->insertItem(tr("Open..."), FILE_OPEN);
    m_fileMenu->insertItem(tr("Open URL..."), FILE_OPEN_URL);
    m_fileMenu->insertItem(tr("Save playlist..."), FILE_SAVE_PLAYLIST);
    m_fileMenu->insertSeparator(FILE_SEPARATOR1);
    m_fileMenu->insertItem(tr("File info..."), FILE_INFO);
    m_fileMenu->insertSeparator(FILE_SEPARATOR2);
    m_fileMenu->insertItem(tr("Preferences..."), FILE_PREFS);

    m_viewMenu = new QPopupMenu(this);
    m_viewMenu->setCheckable(true);
    connect(m_viewMenu, SIGNAL(activated(int)), this, SLOT(viewMenu(int)));
    m_viewMenu->insertItem(tr("Scale to screen"), VIEW_SCALE);
    m_viewMenu->insertItem(tr("Turn LCD Off"), VIEW_LCD_OFF);
    m_viewMenu->insertSeparator(VIEW_SEPARATOR1);
    m_viewMenu->insertItem(tr("Playlist"), VIEW_PLAYLIST);
    m_viewMenu->insertItem(tr("Slider"), VIEW_SLIDER);
    m_viewMenu->insertItem(tr("Album Cover"), VIEW_ALBUM_COVER);

    m_helpMenu = new QPopupMenu(this);
    connect(m_helpMenu, SIGNAL(activated(int)), this, SLOT(helpMenu(int)));
    m_helpMenu->insertItem(tr("About"), 0);

    bar = new QToolBar(this);
    bar->setHorizontalStretchable( TRUE );

    menu = new QPEMenuBar(bar);
    menu->insertItem(tr("File"), m_fileMenu);
    menu->insertItem(tr("View"), m_viewMenu);
    menu->insertItem(tr("Help"), m_helpMenu);

    bar = new QPEToolBar(this);
    info = new QToolButton(Resource::loadPixmap("kino2/info"), tr("File Info"), tr("File Info"), this, SLOT(displayFileInfo()), bar);
    addToPls = new QToolButton(Resource::loadPixmap("kino2/add_to_playlist"),tr("Add to playlist"),tr("Add track to playlist"), this,SLOT(addToPlaylist()), bar);
    addAllToPls = new QToolButton(Resource::loadPixmap("kino2/add_all_to_playlist"),tr("Add all to playlist"),tr("Add all tracks to playlist"), this,SLOT(addAllToPlaylist()), bar);
    delFromPls = new QToolButton(Resource::loadPixmap("kino2/remove_from_playlist"),tr("Remove from playlist"),tr("Remove track from playlist"), this,SLOT(removeFromPlaylist()), bar);
    clearPls = new QToolButton(Resource::loadPixmap("kino2/eraser"),tr("Clear playlist"),tr("Clear playlist"), this,SLOT(clearPlaylist()), bar);

    stack = new QWidgetStack( this );
    stack->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
    setCentralWidget( stack );

    contents = new PlayerBaseImpl( stack );
    stack->addWidget( contents, 0 );
    stack->raiseWidget( contents );

    contents->screen->setBackgroundColor(Qt::black);
    splash = contents->screen->pixmap();

//    contents->screen->show();
//    contents->playlist->resize(contents->screen->size());
//    contents->albumCover->hide();

    // detect orientation
    if (qApp->desktop()->width() > qApp->desktop()->height())
      orientation = Horizontal;
    else
      orientation = Vertical;

    contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_play"));
    contents->stop_button->setEnabled(false);

    up = new QToolButton(Resource::loadPixmap("kino2/up"), tr("Move up"), tr("Move track up in playlist"), contents->playlist, SLOT(up()), bar);
    down = new QToolButton(Resource::loadPixmap("kino2/down"), tr("Move down"), tr("Move track down in playlist"), contents->playlist, SLOT(down()), bar);

    m_columnsMenu = new QPopupMenu(this);
    QStringList list = contents->playlist->columns();
    QStringList::Iterator it = list.begin();

    // skip first two
    it++;
    it++;
   for (; it != list.end(); ++it)
       m_columnsMenu->insertItem(*it);

	connect(m_columnsMenu, SIGNAL(activated(int)), this, SLOT(columnsMenu(int)));

    columns = new QToolButton(Resource::loadPixmap("kino2/playlist"), tr("Columns"), tr("Columns"), 0, "", bar);
    columns->setPopup(m_columnsMenu);
	 columns->setPopupDelay(0);

    connect(contents->playlist, SIGNAL(currentChanged(QListViewItem*)), this, SLOT( currentChanged(QListViewItem*)));
    connect(contents->playlist, SIGNAL(clicked(QListViewItem*)), this, SLOT( clicked(QListViewItem*)));
    connect(contents->playlist, SIGNAL(keyPressed(QKeyEvent *, bool&)), this, SLOT(keyPressed(QKeyEvent *, bool&)));

	contents->start_button->setPixmap(Resource::loadPixmap("kino2/player_start"));
	contents->end_button->setPixmap(Resource::loadPixmap("kino2/player_end"));

    connect( contents->start_button, SIGNAL(clicked()), this, SLOT(startClicked()));
    connect( contents->end_button, SIGNAL(clicked()), this, SLOT(endClicked()));

    connect( contents->play_button, SIGNAL(clicked()), this, SLOT(playClicked()));
    connect( contents->stop_button, SIGNAL(clicked()), this, SLOT(stopClicked()));
    connect( contents->eject_button, SIGNAL(clicked()), this, SLOT(ejectClicked()));
    connect( contents->rev_button, SIGNAL(clicked()), this, SLOT(revClicked()));
    connect( contents->fwd_button, SIGNAL(clicked()), this, SLOT(fwdClicked()));

    process = new OProcess(this);
    connect(process, SIGNAL(processExited(OProcess *)), this, SLOT(processExited(OProcess *)));
    connect(process, SIGNAL(receivedStdout(OProcess *, char *, int)),
    	this, SLOT(receivedStdout(OProcess *, char *, int)));
    connect(process, SIGNAL(receivedStderr(OProcess *, char *, int)),
    	this, SLOT(receivedStderr(OProcess *, char *, int)));
    connect(process, SIGNAL(wroteStdin(OProcess *)), this, SLOT(wroteStdin(OProcess *)));

// there is no need to explicitely connect setDocument() slot
//    channel = new QCopChannel("QPE/Application/kino2", this);
//    connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
//        this, SLOT(setDocument(const QCString&, const QByteArray&)));

    sysChannel = new QCopChannel("QPE/System", this);
    connect(sysChannel, SIGNAL(received(const QCString&, const QByteArray&)),
      this, SLOT(sysChannelReceived(const QCString&, const QByteArray&)));

//    channel = new QCopChannel("QPE/Application/kino2", this);
//    connect(channel, SIGNAL(setDocument(QString)), this, SLOT(setDocument(const QString&)));

//    sysChannel = new QCopChannel("QPE/System", this);
//    connect(sysChannel, SIGNAL((int)), this, SLOT(setBlankLCD(int)));

    m_model = detectModel();
    loadSettings();

    filter = new FileSelectionFilter(kinoPrefs.s_types, this);

    slf = new SlFileSelector("", "", stack, "");
    slf->setFilter(filter);
    connect(slf, SIGNAL(fileSelected(const QFileInfo &)), this, SLOT(fileSelected(const QFileInfo &)));
    connect(slf, SIGNAL(keyPressed(QKeyEvent*, bool&)), this, SLOT(keyPressed(QKeyEvent *, bool&)));
    slf->setListView();
//    slf->setRecentEnabled(true);
//    slf->setRecent(QStringList::split(kinoPrefs.s_history, ","), true);

    connect(qApp, SIGNAL(volumeChanged(bool)), this, SLOT(volumeChanged(bool)));

	wait = new SlDlgWait(this, tr("Loading file(s)..."), true, false);
	wait->hide();

   connect(contents->progress, SIGNAL(sliderPressed()), this, SLOT(sliderPressed()));
   connect(contents->progress, SIGNAL(sliderMoved(int)), this, SLOT(sliderMoved(int)));
	connect(contents->progress, SIGNAL(sliderReleased()), this, SLOT(sliderReleased()));

	if (kinoPrefs.f_saveplaylist)
	{
		QTimer* t = new QTimer(this);
		connect(t, SIGNAL(timeout()), this, SLOT(loadDefaultPlaylist()));
		t->start(1000, true);
	}
}

Player::~Player()
{
    saveSettings();

	if (kinoPrefs.f_saveplaylist)
		savePlaylist(QPEApplication::documentDir() + DEFAULT_PLAYLIST);

    delete process;
}

void Player::setupButtons()
{
	 info->setEnabled(stack->visibleWidget() != slf && contents->playlist->currentItem() != 0);

    addToPls->setEnabled(stack->visibleWidget() == slf && !isAddToPlaylist);
    addAllToPls->setEnabled(stack->visibleWidget() == slf && !isAddToPlaylist);
    delFromPls->setEnabled(contents->playlist->isVisible() && contents->playlist->currentItem() != 0);

    clearPls->setEnabled(contents->playlist->isVisible() && contents->playlist->childCount() > 0);

    up->setEnabled(contents->playlist->isVisible() && contents->playlist->currentItem() != 0);
    down->setEnabled(contents->playlist->isVisible() && contents->playlist->currentItem() != 0);

    columns->setEnabled(contents->playlist->isVisible());
}

void Player::sendCommand(const QString& command)
{
	if (command == "quit\n")
		isManualStop = true;

	process->writeStdin(command, command.length());

	if (kinoPrefs.f_debug)
		qDebug("=>" + command);
}

void Player::setDocument(const QString& file)
{
   if (kinoPrefs.f_debug)
      qDebug("Player::setDocument()");

   isAutoplay = true;
   addToPlaylist(file);

   /*
	QFileInfo fi(file);

	if (fi.exists())
	{
		isAutoplay = true;
		addFile(fi);
//		playClicked();
	}
   */
}

void Player::currentChanged(QListViewItem* item)
{
   if (kinoPrefs.f_debug)
      qDebug("Player::currentChanged()");

	if (item != 0)
	{
		 QString fullName = item->text(0);
	    setSelectedFile(fullName);
	    contents->playlist->ensureItemVisible(item);

      // check which item is selected
      if (positionRestored && kinoPrefs.f_saveplaylist && kinoPrefs.f_file_pos && file_pos > 0)
      {
         if (contents->playlist->currentItemIndex() != kinoPrefs.f_playlist_pos)
         {
            file_pos = 0;  // reset
            positionRestored = false;

            if (kinoPrefs.f_debug)
               qDebug("Different item selected, resetting file_pos");
         }
      }


	}
	else
	{
	    setSelectedFile("");
	}

	setupButtons();
}

void Player::clicked(QListViewItem* item)
{
	if (item != 0 && kinoPrefs.f_playonclick && !isPlaying)
		playClicked();
}

void Player::closeEvent(QCloseEvent *e)
{
    if (isLCDBlank)
    {
//    	isLCDBlank = false;
    	return;
    }

    if (isPlaying)
    {
    	stop();

//		if (isPausing)
//		{
//			resume();
//		    isPausing = false;
//		    sendCommand("pause\n");
//		    contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));
//		}
//		sendCommand("quit\n");
    }
    else if (stack->visibleWidget() == contents)
    {
    	if (process->isRunning())
    		process->kill();

		e->accept();
    }
    else
    {
		stack->raiseWidget(contents);
		setupButtons();
    }
}

 void Player::setSelectedFile(const QString& selectedFile)
 {
     if (selectedFile == "")
     {
         this->selectedFile = "";
         setCaption(tr("Kino2"));
         return;
     }

     if (selectedFile.startsWith("http://"))
     {
         setCaption(selectedFile + " - " + tr("Kino2"));
         this->selectedFile = selectedFile;
         return;
     }

     QFileInfo fi(selectedFile);
     setCaption(fi.baseName() + " - " + tr("Kino2"));
     this->selectedFile = fi.absFilePath();
 }

void Player::fileSelected(const QFileInfo &fileInfo)
{
   if (kinoPrefs.f_debug)
   {
      qDebug("Player::fileSelected() - " + fileInfo.filePath());
      qDebug("slf->currentFile() - " + slf->currentFile());
   }

	if (!kinoPrefs.f_addontap)
		return;

   if (addToPlaylist(slf->currentFile()))
   {
       closeFileSelector = true;
       //stack->raiseWidget(contents); !!!!!!!!!!!!!!!!!!!
       setupButtons();
   }
}

void Player::processExited(OProcess* /* proc */)
{
	if (kinoPrefs.f_debug)
	    qDebug("processExited()...");

   wait->hide();

   if (kinoPrefs.f_keyboard)
   	releaseKeyboard();

	toggleSuspend(true);

   if (kinoPrefs.f_lid_suspend)
      toggleLidSuspend(saved_lid_suspend);

	isIgnoreOutput = false;

	// CPU restore
	if (kinoPrefs.f_cpu && !isIdentify && !isAddToPlaylist)
	{
		if (freq.clock != get_freq().clock)
			set_freq(freq);
	}

	if (isLCDBlank /* kinoPrefs.f_lcd_off */)
		lcdOn();

    if (isIdentify)
    {
		QStringList listUtf8 = QStringList::split("\n", bufferUtf8);
		QStringList::Iterator it = listUtf8.begin();

		QStringList listLocal = QStringList::split("\n", bufferLocal);
		QStringList::Iterator itLocal = listLocal.begin();

		for (; it != listUtf8.end(); ++it, ++itLocal)
	    {
			QString str = (*it).stripWhiteSpace();
			QString strLocal = (*itLocal).stripWhiteSpace();

			if (!str.startsWith("A:"))	// this somehow crashes the app
				if (kinoPrefs.f_debug)
					qDebug(str.utf8());

			if (str.startsWith("Playing"))		// next file
			{
				QString file = str.mid(7).stripWhiteSpace();
				if (file[file.length() - 1] == '.')
					file.truncate(file.length() - 1);

				if (kinoPrefs.f_debug)
					qDebug(file + " selected.");

				setSelectedFile(file);	// not yet added to playlist
				continue;
			}

			// here we should have all media info populated
			if (str.startsWith("Starting playback..."))
			{
				QListViewItem* item = contents->playlist->firstChild();
				bool found = false;
				while (item)
				{
					if (selectedFile == item->text(0))
					{
						// select new file in playliat
						contents->playlist->setCurrentItem(item);
						found = true;
						break;
					}

					item = item->itemBelow();
				}

				if (!found)
				{
					contents->playlist->addToPlaylist(selectedFile, minfo[selectedFile]);
					if (kinoPrefs.f_debug)
						qDebug(selectedFile + " added.");

               if (slf->isVisible())
               {
                  // restore current selection
                  if (kinoPrefs.f_debug)
                     qDebug("Restore selected file: - " + saveCurrentFile);

                  slf->setCurrentFile(saveCurrentFile);
                  slf->setSelected(saveCurrentFile);

                  if (kinoPrefs.f_debug)
                     qDebug("Current File is: - " + slf->currentFile());

                  if (closeFileSelector)
                  {
                     closeFileSelector = false;
                     stack->raiseWidget(contents);
                  }
                  else
                     // move to the next item
                     slf->setSelectedNext(true);
               }

					setupButtons();
				}

				continue;
			}

			// try to parse media info by default
			parseMediaInfo(selectedFile, str, strLocal);
		}

		isIdentify = false;
	}

	if (isAddToPlaylist)
		isAddToPlaylist = false;

	setupButtons();
	stop();

//	contents->screen->setBackgroundColor(Qt::black);

//	if (splash != 0)
//		contents->screen->setPixmap(*splash);

	isPlaying = false;
	isPausing = false;

	if (isSelectFirst)
	{
		isSelectFirst = false;

      if (kinoPrefs.f_playlist_pos /* && playlist_pos > 0 */)
      {
         contents->playlist->setCurrentItemIndex(playlist_pos);
         positionRestored = true;
      }
      else
         contents->playlist->setCurrentItemIndex(0);

//		QListViewItem* item = contents->playlist->firstChild();
//		contents->playlist->setCurrentItem(item);
      QListViewItem* item = contents->playlist->currentItem();
      if (item > 0)
   		setSelectedFile(item->text(0));
	}

   // refresh the screen
   if (needToRefresh)
   {
      needToRefresh = false;
      QCopEnvelope("QPE/System", "applyStyle()");
   }

	// need to play file immidiately when invoked with setDocument()
	if (isAutoplay)
	{
		isAutoplay = false;
		playClicked();
	}
}

void Player::sliderPressed()
{
   // reset jump position
   jumpTo = 0;
}

void Player::sliderMoved(int value)
{
   // remember slider posision
   jumpTo = value;
}

void Player::sliderReleased()
{
   // seack by % works in mplayer 1.1.4 and above
   seek(jumpTo, 2);
}

void Player::receivedStdout(OProcess* /* proc */, char* buffer, int buflen)
{
	// have to use C functions here because buffer crashes QString sometimes...
	char b[buflen + 1];
	strncpy(b, buffer, buflen);

//   if (strstr(b, "Exiting") != 0)
//   {
//      // refresh the screen
//      if (needToRefresh)
//         QCopEnvelope("QPE/System", "applyStyle()");
//   }

	if (isIgnoreOutput && strstr(b, "Playing") == 0)	// next file
		return;

	isIgnoreOutput = false;

	// split buffer in case we recieve several lines
	QString sUtf8 = QString::fromUtf8(buffer, buflen);
	QString sLocal = QString::fromLocal8Bit(buffer, buflen);
	QTextCodec* codec = QTextCodec::codecForName(kinoPrefs.s_enc_mp3);
	if (codec != 0)
		sLocal = codec->toUnicode(buffer, buflen);

	// handled when process finishes
	if (isIdentify)
	{
		bufferUtf8 += sUtf8;
		bufferLocal += sLocal;
		return;
	}

	QStringList listUtf8 = QStringList::split("\n", sUtf8);
	QStringList::Iterator it = listUtf8.begin();

	QStringList listLocal = QStringList::split("\n", sLocal);
	QStringList::Iterator itLocal = listLocal.begin();

	for (; it != listUtf8.end(); ++it, ++itLocal)
    {
		QString str = (*it).stripWhiteSpace();
		QString strLocal = (*itLocal).stripWhiteSpace();

		if (!str.startsWith("A:"))	// this somehow crashes the app
			if (kinoPrefs.f_debug)
				qDebug(str.utf8());

		if (str.startsWith("Playing"))		// next file
		{
         // refresh the screen from previousely played video file
         if (needToRefresh)
         {
            needToRefresh = false;
            QCopEnvelope("QPE/System", "applyStyle()");
         }

			QString file = str.mid(7).stripWhiteSpace();
			if (file[file.length() - 1] == '.')
				file.truncate(file.length() - 1);

			setSelectedFile(file);	// not yet added to playlist
			continue;
		}

		int track = 0;

		// here we should have all media info populated
		if (str.startsWith("Starting playback..."))
		{
         if (kinoPrefs.f_fade)
            fadeIn();

			wait->hide();

         // seek in current file
         if (positionRestored && kinoPrefs.f_saveplaylist && kinoPrefs.f_file_pos && file_pos > 0)
         {
            if (kinoPrefs.f_debug)
               qDebug("Seeking to: " + QString::number(file_pos));

            seek(file_pos, 2);  // absolute
            file_pos = 0;  // reset
            positionRestored = false;
         }

			QListViewItem* item = contents->playlist->firstChild();
//			bool found = false;
			while (item)
			{
				track++;
				if (selectedFile == item->text(0))
				{
					// select new file in playliat
					contents->playlist->setCurrentItem(item);
//					found = true;
					break;
				}

				item = item->itemBelow();
			}

			DocLnk doc(selectedFile);
			isVideoFile = doc.type().startsWith("video/") || minfo[selectedFile].VIDEO_FORMAT != "";

         if (isVideoFile && isLCDBlank)
            lcdOn();

         if (kinoPrefs.f_lcd_off && !isVideoFile && !isLCDBlank)
            lcdOff();

         // should we also refresh screen on other models?
         // under what conditions???
         needToRefresh = !isIdentify && isVideoFile /* && (m_model == SL_C3000) */ && useScale /* && minfo[selectedFile].VIDEO_WIDTH <= 320 */;

			// ignore all mlayer output for video files or if tracking is disabled by user
			isIgnoreOutput = isVideoFile && !isIdentify || !kinoPrefs.f_tracking;

			// CPU change
			if (kinoPrefs.f_cpu)
			{
				ClockVCore new_freq = isVideoFile ? kinoPrefs.s_v_cpu : kinoPrefs.s_a_cpu;
				if (new_freq.clock != get_freq().clock)
					set_freq(new_freq);
			}

			// setup progres sindicator
			int length = minfo[selectedFile].LENGTH;

			if (kinoPrefs.f_debug)
				qDebug("Length: " + QString::number(length));

			contents->progress->setRange(0, length);
			contents->progress->setLineStep(1);
			contents->progress->setPageStep(1);
			contents->progress->setTickInterval(1);

			QString mode = tr("Normal");
			if (kinoPrefs.f_repeat)
			    mode = tr("Repeat");
			else if (kinoPrefs.f_random)
			    mode = tr("Random");

			// set text
			QString text =
				tr("Artist: ") + minfo[selectedFile].Artist + "\n" +
				tr("Album: ") + minfo[selectedFile].Album + "\n" +
				tr("Title: ") + minfo[selectedFile].Title + "\n" +
				tr("Track: ") + QString::number(track) + "/" + QString::number(contents->playlist->childCount()) + "\n" +
				tr("Mode: ") + mode;

			contents->screen->setFrameStyle(QFrame::Box | QFrame::Sunken);
			contents->screen->setFont(QFont(kinoPrefs.s_font, kinoPrefs.s_fontSize, QFont::Bold));
			contents->screen->setBackgroundColor(Qt::white);
			contents->screen->setText(text);

         displayCoverImage();

			continue;
		}

      if (str.startsWith("PTS=") && isPlaying && !isVideoFile)
      {
         currentPosition = str.mid(4).toInt();
         contents->time->setText(length(currentPosition) + "/" + length(minfo[selectedFile].LENGTH));
         contents->progress->setValue(currentPosition);
      }
    }
}

void Player::receivedStderr(OProcess* process, char *buffer, int buflen)
{
	QString str = QString::fromUtf8(buffer, buflen);
   // always report errors...
//	if (kinoPrefs.f_debug)
	    qDebug(str.utf8());

   if (errorTimeout == 0)
   {
      errorTimeout = time(0);
      errorCount = 1;
      return;
   }

   // terminate process when too many errors are recieved for a short period
   errorCount++;
   if ((time(0) - errorTimeout) > 1)
   {
      if (errorCount > 1)
      {
         qDebug("Too many errors from mplayer process, terminating...");
//         needToRefresh = false;
         delayedStop();

         if (process->isRunning())
            if (!process->kill())
               process->kill(SIGKILL);

//         QMessageBox::warning(0, tr("Kino 2"), tr("Too many errors recieved from mplayer.\n"
//            "Mplayer process was terminated."));

         return;
      }

      errorTimeout = 0; // restart
   }
}

void Player::wroteStdin(OProcess* /* proc */)
{
	if (kinoPrefs.f_debug)
    	qDebug("wroteStdin()...");
}

void Player::identifyFile(const QString& file)
{
	identifyFile(QStringList(file));
}

void Player::identifyFile(const QStringList& files)
{
   if (kinoPrefs.f_debug)
      qDebug("Player::identifyFile()");

	if (files.count() == 0)
		return;

	QStringList args;
	args << MPLAYER;
	args << "-identify"	<< "-vo" << "null" << "-ao" << "null";
	args << "-v";		// verbose, to get OGG properties
   args << "-nortc";
   args << "-ac" << "-mp3,";     // disable mp3lib codec, it's not compiled into mplayer

	// this option is to workaround a stupid problem with mplayer 1.x versions,
	// which would play the file after identifying...
	args << "-frames";
	args << "0";

   isIdentify = true;
	isIgnoreOutput = false;

	process->clearArguments();
	*process << args;

	QStringList::ConstIterator it = files.begin();
	for (; it != files.end(); ++it)
		*process << (*it).utf8();

	bufferUtf8 = "";
	bufferLocal = "";

	wait->show();

   errorTimeout = 0;
   errorCount = 0;

	process->start(OProcess::NotifyOnExit, OProcess::All);
}

void Player::showSlider(bool visible)
{
   contents->setUpdatesEnabled(false);
	if (visible)
	{
		contents->progress->show();
		contents->time->show();
//		contents->sliderLayout->show();
	}
	else
	{
		contents->progress->hide();
		contents->time->hide();
//		contents->sliderLayout->hide();
	}

	m_viewMenu->setItemChecked(VIEW_SLIDER, visible);
   contents->setUpdatesEnabled(true);
}

void Player::playClicked()
{
	isIdentify = false;

	if (isPlaying)
	{
		if (isPausing)
		{
	    	resume();
	//		isPausing = false;
	//		sendCommand("pause\n");
	//		contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));
			return;
		}
		else
		{
	    	pause();
	//		isPausing = true;
	//		sendCommand("pause\n");
	//		contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_play"));
			return;
		}
	}
    else if (selectedFile != "")
    {
		QStringList args;
		QString buf_geom;
		QString vop_args;
		QString s_cache;
		QString s_band;

		isIgnoreOutput = false;

	//	SlMisc::enableAutoPowerOff(false, false, false);

		args << MPLAYER;
		args << "-slave";
		args << "-nortc";
		//args << "-v";		// verbose
		//args << "-really-quiet";
      args << "-quiet";
		args << "-ac" << "-mp3,";		// disable mp3lib codec, it's not compiled into mplayer

      // more presice seeking, but can be slow
      if (kinoPrefs.f_hrseek)
         args << "-hr-mp3-seek";

      // always enable volume audio filter
      if (kinoPrefs.f_swvolume)
         args << "-softvol" << "-softvol-max" << QString::number(kinoPrefs.s_maxvolume);

//      if (kinoPrefs.f_fade)
//        args << "-af" << "volume=0";

//      if (kinoPrefs.f_fade)
//         args << "-af" << "volume=-50";

		if (kinoPrefs.f_noaspect)
			args << "-noaspect";

		if (kinoPrefs.f_repeat)
		    args << "-loop" << "0";
		else if (kinoPrefs.f_random)
		    args << "-shuffle";

		if (kinoPrefs.s_priority > 0)
		    args << "-rtprio" << QString::number(kinoPrefs.s_priority);

		if (kinoPrefs.f_index)
			args << "-idx";

		// always use overlay
		args << "-vo";
		switch (m_model)
		{
			case SL_C7X0_860:
				args << "w100";
				break;

			case SL_C3000:
				args << "bvdd";
				break;

			case SL_5600:
			case SL_6000:
			case UNSUPPORTED:
			default:
				args << "fbdev";
		}

	    if (kinoPrefs.f_doublebuffer)
			args << "-double";
       else
         args << "-nodouble";

//		if (kinoPrefs.f_directrendering)
		    args << "-dr";

		if (kinoPrefs.f_dropframes)
		    args << "-framedrop";

       // autosync should fix problem with framedrops on high-bitrate videos
       if (kinoPrefs.f_autosync /* && minfo[selectedFile].VIDEO_BITRATE > 600 */)
          args << "-autosync" << "100";

		if (kinoPrefs.f_cache)
		{
			if (kinoPrefs.s_cache > 0)
			{
			    args << "-cache";
			    s_cache = QString::number(kinoPrefs.s_cache);
			    args << s_cache;
			}
		}
		else
			args << "-nocache";

		if (kinoPrefs.s_bandwidth > 0)
		{
		    args << "-bandwidth";
		    s_band = QString::number(kinoPrefs.s_bandwidth);
		    args << s_band;
		}

		if (kinoPrefs.f_sdl)
			args << "-ao" << "sdl";

		if (kinoPrefs.f_osd)
			args << "-osdlevel" << "3";

		// add items from current item
		QListViewItem* item = contents->playlist->currentItem();
		while(item)
		{
			QString file = item->text(0);
			addPlayitem(args, file);
			item = item->itemBelow();
		}

		// add items before current item
		item = contents->playlist->firstChild();
		while(item != contents->playlist->currentItem())
		{
			QString file = item->text(0);
			addPlayitem(args, file);
			item = item->itemBelow();
		}

		QString s;
		process->clearArguments();
		for (QStringList::Iterator it = args.begin(); it != args.end(); it++)
		{
			s.append(*it).append(" ");
			*process << (*it).utf8();

		}

		if (kinoPrefs.f_debug)
			qDebug(QString("Starting mplayer with options: ") + s);

//		if (kinoPrefs.f_external_mem)
//			process->setEnvironment("MPLAYER_W100_USEEXTVRAM", "1");

		toggleSuspend(false);

      if (kinoPrefs.f_lid_suspend)
         toggleLidSuspend(false);

      if (kinoPrefs.f_keyboard)
   		grabKeyboard();

		// save current frequency
		freq = get_freq();

      if (kinoPrefs.f_fade)
      {
         sysVolume = readSystemVolume();
         set_volume(0);
         QTimer::singleShot(1300, this, SLOT(restoreSysVolume()));
      }

      errorTimeout = 0;
      errorCount = 0;

		process->start(OProcess::NotifyOnExit, OProcess::All);

      // start with zero volume if fade effects are enabled
      if (kinoPrefs.f_fade)
         sendCommand("volume 0 1\n");

      imageIndex = 0;
      isManualStop = false;
      isPlaying = true;
      isPausing = false;

      contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));
      contents->stop_button->setEnabled(true);

		// skip to the currently selected item
//		if (pos > 0)
//		{
////			sendCommand("pause\n");
//			sendCommand(QString("pt_step +") + QString::number(pos) + "\n");
////			sendCommand("pause\n");
//		}
    }
}

void Player::addPlayitem(QStringList& args, QString file)
{
	args << file;

	DocLnk doc(selectedFile);
	isVideoFile = doc.type().startsWith("video/") || minfo[selectedFile].VIDEO_FORMAT != "";

	if (isVideoFile)
	{
		// resample audio only for video files when needed
		if (kinoPrefs.f_resample && minfo[selectedFile].AUDIO_RATE != 44100)
			args << "-af" << "lavcresample=44100";

		int width = minfo[selectedFile].VIDEO_WIDTH;
		int height = minfo[selectedFile].VIDEO_HEIGHT;

      // use hardware rotation for bvdd and w100 driver
		bool rotate = kinoPrefs.f_rotate && (width > height) && m_model != SL_C3000 && m_model != SL_C7X0_860;

      if (rotate && !kinoPrefs.f_smart_crop)
		    args << "-vf" << "rotate=1";

		if (kinoPrefs.f_smart_crop)
		{
			int widths[] = {/*160,*/ 320, 640};
			int heights[] = {/*120,*/ 240, 480};

			int cropW = width, cropH = height;

			if (width > 0 && height > 0)
			{
				for (int i = 1; i >= 0; i--)
				{
					if (width >= widths[i])
					{
						cropW = widths[i];
						break;
					}
				}

				for (int i = 1; i >= 0; i--)
				{
					if (height >= heights[i])
					{
						cropH = heights[i];
						break;
					}
				}

				if (cropW != width || cropH != height)
				{
					args << "-vf";

               // use hardware rotation for bvdd driver
					if (rotate)
						args << QString().sprintf("crop=%d:%d, rotate=1", cropH, cropW);
					else
						args << QString().sprintf("crop=%d:%d", cropW, cropH);
				}
//					args << "-vf" << QString(kinoPrefs.f_rotate ? "rotate=1," : "") + "crop=" + s_width + ":" + s_height;
			}
		}

      if (useScale)
      {
         // scale to screen size
         if (kinoPrefs.f_softscale)
         {
//               args << "-vf-add" << "rotate=1";
            args << "-vf-add" << QString().sprintf("scale=%d:%d", qApp->desktop()->width(), qApp->desktop()->height());
         }
         else
         {
            if (m_model == SL_C3000)
               args << "-vm";

            if (m_model == SL_C7X0_860)
               args << "-fs";

            // resize to QVGA, so that mplayer can scale automaticaly
            if (m_model == SL_C3000 || m_model == SL_C7X0_860)
            {
               if (width < 320 || height < 240)
               {
                  int scaleW = 320, scaleH = 240;
                  if (((float)width / (float)height) > 1.6)    // 16:9
                     scaleH = 176;

                  args << "-vf-add" << QString().sprintf("scale=%d:%d", scaleW, scaleH);
               }
            }
         }
//            if (m_model == SL_6000)
//               args << "-vf-add" << "rotate=1" << "-vf-add" << "scale=480:640";
      }
	}

	// add user-defined options to each file
	if (kinoPrefs.s_mplayeroptions != "")
	{
		QStringList list = QStringList::split(' ', kinoPrefs.s_mplayeroptions);
		QStringList::Iterator it;
		for(it = list.begin(); it != list.end(); args << *it, it++);
	}
}

void Player::toggleSuspend(bool enable)
{
    QCopEnvelope e("QPE/System", "setScreenSaverIntervals(int,int,int)" );
    if (enable)
    {
    	if (kinoPrefs.f_debug)
		    qDebug("Enabling suspend with %d %d %d\n", (dim > 0 ? intervalDim : 0), intervalLightOff, intervalSuspend);

	    e << intervalDim << intervalLightOff << intervalSuspend;
    }
	else
	{
		if (kinoPrefs.f_debug)
	    	qDebug("Disabling suspend\n");

	    e << 0 << 0 << 0;

       // also try these:
       // setScreenSaverMode(int), setSuspendOnOff(bool), setScreenSaverOnOff(bool), setScreenSaverInterval(int)
    }
}

void Player::toggleLidSuspend(bool disable)
{
   Config qpeconfig("qpe");
   qpeconfig.setGroup("Screensaver");

   qpeconfig.writeEntry("CoverSuspend", disable);
}

void Player::stopClicked()
{
	stop();
}

void Player::startClicked()
{
    if (isPlaying)
    {
		if (isPausing)
		{
			resume();
//		    isPausing = false;
//		    sendCommand("pause\n");
//		    contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));
		}

      if (kinoPrefs.f_fade)
      {
         // schedule actual pause in 1.1 sec
         QTimer::singleShot(1100, this, SLOT(delayedPrev()));
         fadeOut();
      }
      else
         delayedPrev();
    }
}

void Player::delayedPrev()
{
   sendCommand("pt_step -1\n");
}

void Player::endClicked()
{
    if (isPlaying)
    {
		if (isPausing)
		{
			resume();
//		    isPausing = false;
//		    sendCommand("pause\n");
//		    contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));
		}

      if (kinoPrefs.f_fade)
      {
         // schedule actual pause in 1.1 sec
         QTimer::singleShot(1100, this, SLOT(delayedNext()));
         fadeOut();
      }
      else
         delayedNext();
    }
}

void Player::delayedNext()
{
   sendCommand("pt_step +1\n");
}

void Player::pause()
{
   if (kinoPrefs.f_fade)
   {
      // schedule actual pause in 1.1 sec
      QTimer::singleShot(1100, this, SLOT(delayedPause()));
      fadeOut();
   }
   else
      delayedPause();
}

void Player::delayedPause()
{
   sendCommand("pause\n");
   isPausing = true;
   contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_play"));

   // enable suspend?
   if (kinoPrefs.f_suspend)
      toggleSuspend(true);
}

void Player::resume()
{
    sendCommand("pause\n");
    isPausing = false;
    contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));

   // disable suspend?
   if (kinoPrefs.f_suspend)
      toggleSuspend(false);

   if (kinoPrefs.f_fade)
      fadeIn();
}

void Player::stop()
{
    if (isPausing)
    {
    	sendCommand("pause\n");
    	isPausing = false;

      QTimer::singleShot(50, this, SLOT(stopClicked()));
		return;
    }

    if (isPlaying)
    {
      if (kinoPrefs.f_fade)
      {
         // schedule actual stop in 1.1 sec
         QTimer::singleShot(1100, this, SLOT(delayedStop()));
         fadeOut();
      }
      else
         delayedStop();
    }
}

void Player::delayedStop()
{
   sendCommand("quit\n");
   isPlaying = false;
   contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_play"));
   contents->stop_button->setEnabled(false);
}

void Player::seek(int value, int type /* = 0 */)
{
	QString command;

    if (isPlaying)
    {
		if (isPausing)
		{
			resume();
//		    isPausing = false;
//		    sendCommand("pause\n");
//		    contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));
		}

		if (kinoPrefs.f_debug)
			qDebug("seek(): value=" + QString::number(value) + " " + QString::number(type));

/*
		int length = minfo[selectedFile].LENGTH;
		if (type == 2)	// absolute
		{
			if (length > 0 && value > length)
				value = length;

			// calculate relative position
			if (currentPosition > 0)
				value = value - currentPosition;
		}
*/

//		sendCommand(command.sprintf("seek %d\n", value));
		sendCommand(command.sprintf("seek %d %d\n", value, type));
    }
}

void Player::volumeUp()
{
   if (!kinoPrefs.f_swvolume)
	   volumePercent = readSystemVolume();

    setVolume(volumePercent + kinoPrefs.s_volstep);
}

void Player::volumeDown()
{
   if (!kinoPrefs.f_swvolume)
	  volumePercent = readSystemVolume();

	setVolume(volumePercent - kinoPrefs.s_volstep);
}

void Player::setVolume(int percent)
{
    // clamp volume percent to be between 0 and 100
    volumePercent = (percent < 0) ? 0 : ((percent > 100) ? 100 : percent);

    if (kinoPrefs.f_swvolume)
       setSoftwareVolume(volumePercent);
    else
       writeSystemVolume(volumePercent);
}

void Player::setSoftwareVolume(int percent)
{
   // calculate current volume level based on maxvolume
   int volume = (kinoPrefs.s_maxvolume * percent) / 100;

   if (volume < 0)
      volume = 0;

   if (volume > kinoPrefs.s_maxvolume)
      volume = kinoPrefs.s_maxvolume;

   QString command;
   sendCommand(command.sprintf("volume %d 1\n", volume));
}

void Player::restoreSysVolume()
{
   // restore system hardware volume
   if (sysVolume > 0)
      set_volume(sysVolume);
}

void Player::fadeOut()
{
   // initialize
   volumeStep = volumePercent / 5;
   volumeFade = volumePercent;

   QTimer::singleShot(100, this, SLOT(fadeOutStep()));
}

void Player::fadeOutStep()
{
   volumeFade -= volumeStep;
   setSoftwareVolume(volumeFade);

   if (volumeFade > 0)
      QTimer::singleShot(100, this, SLOT(fadeOutStep()));
}

void Player::fadeIn()
{
   // initialize
   volumeStep = volumePercent / 5;
   volumeFade = 0;

   QTimer::singleShot(100, this, SLOT(fadeInStep()));
}

void Player::fadeInStep()
{
   volumeFade += volumeStep;
   setSoftwareVolume(volumeFade);

   if (volumeFade < volumePercent)
      QTimer::singleShot(100, this, SLOT(fadeInStep()));
}

void Player::volumeChanged(bool nowMuted)
{
   if (kinoPrefs.f_debug)
      qDebug("volumeChanged()");

   // ignore when software volmue control is enabled
   if (kinoPrefs.f_swvolume)
      return;

    if (!nowMuted)
        volumePercent = readSystemVolume();

    // Handle case where muting it toggled
    if (muted != nowMuted)
		muted = nowMuted;
}

void Player::mute(bool toggled)
{
   muted = toggled;

   if (kinoPrefs.f_swvolume)
      sendCommand("mute\n");
   else
      writeSystemVolume(volumePercent);
}

void set_freq(ClockVCore frequency)
{
   qDebug("Set frequency: clock = " + QString::number(frequency.clock) + ", vcore = " + QString::number(frequency.vcore));

	if (frequency.clock <= 0)
		return;

   bool isPXA270 = false;
   QFile fCPU("/proc/cpuinfo");
   if(fCPU.open(IO_ReadOnly))
   {
      QString s;
      fCPU.readLine(s, 100);
      isPXA270 = s.find("PXA270") != -1;
      fCPU.close();
   }

   // set voltage
   QString s_freq;
   if (frequency.vcore > 0 && isPXA270)
      system(s_freq.sprintf("sudo sh -c \"echo %02X > /proc/zaurus/VCORE\"", frequency.vcore));

   if (isPXA270)
      system(s_freq.sprintf("sudo sh -c \"echo %08X > /proc/zaurus/CCCR\"", frequency.clock));
   else
	   system(s_freq.sprintf("sudo sh -c \"echo %08X > /proc/cccr_change\"", frequency.clock));
}


ClockVCore get_freq()
{
	FILE *fp;
   ClockVCore freq;

   bool isPXA270 = false;
   QFile fCPU("/proc/cpuinfo");
   if(fCPU.open(IO_ReadOnly))
   {
      QString s;
      fCPU.readLine(s, 100);
      isPXA270 = s.find("PXA270") != -1;
      fCPU.close();
   }

   if (isPXA270)
      fp = fopen("/proc/zaurus/CCCR", "r");
   else
   	fp = fopen("/proc/cccr_change", "r");

	if (fp != 0)
   {
   	fscanf(fp, "%x", &freq.clock);
   	fclose(fp);
   }

   if (isPXA270)
   {
      fp = fopen("/proc/zaurus/VCORE", "r");

      if (fp != 0)
      {
         fscanf(fp, "%x", &freq.vcore);
         fclose(fp);
      }
   }

   qDebug("Get frequency: clock = " + QString::number(freq.clock) + ", vcore = " + QString::number(freq.vcore));
	return freq;
}

int set_volume(int value)
{
	int new_vol;

	if (value > 100) value = 100;
	if (value < 0)   value = 0;

	new_vol = value | (value << 8);

	int mixerfd = ::open("/dev/mixer", O_RDWR);
	if (mixerfd < 0)
	{
		qDebug("Unable to open mixer device.");
		return -1;
	}

	int devmask;
	if(::ioctl(mixerfd, SOUND_MIXER_READ_DEVMASK, &devmask) == -1)
	{
		qDebug("Unable to open mixer device.");
		::close(mixerfd);
		return -1;
	}

	if (::ioctl(mixerfd, SOUND_MIXER_WRITE_VOLUME, &new_vol) == -1)
    	qDebug("Unable to set volume.");

	::close(mixerfd);
	return value;
}

int get_volume(void)
{
  	int orig_vol;

	int mixerfd = ::open("/dev/mixer", O_RDWR);
	if (mixerfd < 0)
	{
		qDebug("Unable to open mixer device.");
		return -1;
	}

	int devmask;
	if(::ioctl(mixerfd, SOUND_MIXER_READ_DEVMASK, &devmask) == -1)
	{
		qDebug("Unable to open mixer device.");
		::close(mixerfd);
		return -1;
	}

	if (::ioctl(mixerfd, SOUND_MIXER_READ_VOLUME, &orig_vol) == -1)
	{
		qDebug("Unable to read volume.");
		::close(mixerfd);
		return -1;
	}

    ::close(mixerfd);
   	return ( (orig_vol & 0xff) + ((orig_vol >> 8) & 0xff) ) / 2;
}

int Player::readSystemVolume()
{
    Config cfg("qpe");
    cfg.setGroup("Volume");

    muted = cfg.readBoolEntry("VolumeMute");
    if (muted)
	    return cfg.readNumEntry("VolumeRightPercent");
	 else
    	 return get_volume();
}

void Player::writeSystemVolume(int volumePercent)
{
	Config cfg("qpe");
	cfg.setGroup("Volume");
	cfg.writeEntry("VolumePercent", volumePercent);
	cfg.writeEntry("VolumeRightPercent", volumePercent);

    if (kinoPrefs.f_debug)
	    qDebug(QString("Setting volume: ") + QString::number(volumePercent) + ", muted: " + QString::number((int)muted));

    set_volume(muted ? 0 : volumePercent);

	 QCopEnvelope( "QPE/System", "volumeChange(bool)" ) << muted;
}


void Player::revClicked()
{
	seek(-10);
}

void Player::fwdClicked()
{
	seek(10);
}

void Player::ejectClicked()
{
    if (isPlaying)
    {
		if (isPausing)
		{
			resume();
//		    isPausing = false;
//		    sendCommand("pause\n");
//		    contents->play_button->setPixmap(Resource::loadPixmap("kino2/player_pause"));
		}
		sendCommand("quit\n");
    }

    stack->raiseWidget(slf);
    slf->setFocus();
    setupButtons();
}

Player::ACTIONS Player::lookupActionForKey(int key)
{
   ACTIONS result = ACTION_NONE;

	// cancel key always stops
	if (key == Key_Escape)
	{
      if (isLCDBlank)
      {
          if (kinoPrefs.f_debug)
             qDebug("Player::lookupActionForKey() - LCD is off, ignoring Cancel key.");

         isLCDBlank = false;     // need to set this manualy
         result = ACTION_NONE;
      }
      else
   		result = ACTION_PLAY_STOP;
	}
	// lid closed/opened
	else if (key == 0x2009 && isPlaying && isVideoFile && kinoPrefs.f_lid)
   {
		result = ACTION_PAUSE_RESUME;
   }
	// to avoid changing volume change
	else if ((key == Key_Up || key == Key_Down) && (contents->playlist->isVisible() || stack->visibleWidget() == slf) && !isPlaying)
   {
		result = ACTION_NONE;
   }
   else
   {
   	for (int action = ACTION_NONE; action < ACTION_LAST; action++)
   	{
         if (doubleKey)
         {
             if (kinoPrefs.f_debug)
                qDebug("Player::lookupActionForKey() - Processing doubleKey.");

//            if (kinoPrefs.remoteActions[action] == key + RemoteControlButton::KEY_DOUBLE)
            if (((kinoPrefs.keyboardActions[action] == key + RemoteControlButton::KEY_DOUBLE) || (kinoPrefs.remoteActions[action] == key + RemoteControlButton::KEY_DOUBLE))
               && (savedKey == key))   // must be the same key
            {
                if (kinoPrefs.f_debug)
                   qDebug("Player::lookupActionForKey() - found double key.");

               doubleKey = false;      // double key was handled, need to clear flag here
               result = (Player::ACTIONS)action;
               break;
            }
         }
   		else
         {
            if (kinoPrefs.keyboardActions[action] == key || kinoPrefs.remoteActions[action] == key)
            {
               if (kinoPrefs.f_debug)
                  qDebug("Player::lookupActionForKey() - found single key.");

               result = (Player::ACTIONS)action;
               break;
            }
         }
   	}
   }

   // clear double key flag in any case
   doubleKey = false;
	return result;
}

void Player::keyPressed(QKeyEvent* event, bool& /* accepted */)
{
   if (kinoPrefs.f_debug)
      qDebug(QString("keyPressed(): 0x") + QString::number(event->key(), 16) + " "+ QString::number(event->ascii()) + (" \"") + event->text() + "\"");

   if (/* slf->isVisible() && */ event->key() == Key_Return)
      return;

//   if (event->key() != Key_Return)  // avoid duplicates
   {
      if (kinoPrefs.f_debug)
         qDebug("Passing event down to keyPressEvent()");

   	keyPressEvent(event);
   }
}

void Player::keyPressEvent(QKeyEvent *event)
{
   int key = event->key();
   savedKey = 0;

	if (kinoPrefs.f_debug)
		qDebug(QString("keyPressEvent(): 0x") + QString::number(key, 16) + " "+ QString::number(event->ascii()) + (" \"") + event->text() + "\"");

   if (slf->isVisible() && key == Key_Return)
      return;

	if (isPlaying /* && !isVideoFile */)  // seeking by % works now
	{
		if (key >= Key_0 && key <= Key_9)
		{
			int value = (key - Key_0) * 10;
         seek(value, 1);      // seek absolute by %
			return;
		}
	}

	if (key > 0)
	{
      // handle double click only for remote and during normal playback
      if (!doubleKey && isPlaying && !isIdentify /* && (key >= RemoteControlButton::KEY_PLAY_PAUSE && key <= RemoteControlButton::KEY_VOLUME_DOWN) */)
      {
         if (key != Key_Escape)
         {
            if (kinoPrefs.f_debug)
                qDebug("Starting double key timer.");

            // start double key timer
            doubleKey = true;
            savedKey = key;
            QTimer::singleShot(300, this, SLOT(doubleKeyTimeout()));      // 0.3 sec

            // do not process anything here untill timer expires
            return;
         }
      }

      if (processKey(key))
         return;
	}

   if (kinoPrefs.f_debug)
      qDebug("event->ignore()");

   // let parent control process this event
   event->ignore();
}

void Player::doubleKeyTimeout()
{
    if (kinoPrefs.f_debug)
       qDebug("Player::doubleKeyTimeout()");

   // timer has expired but second key was not press,
   // clear double key flas and proceess saved key
   if (doubleKey)
   {
       doubleKey = false;

       if (kinoPrefs.f_debug)
          qDebug("Processing saved key.");

      int key = savedKey;
      savedKey = 0;        // clear saved key before processing accurs
      processKey(key);
   }
}

bool Player::processKey(int key)
{
    switch (lookupActionForKey(key))
    {
//       case Key_Enter:
//       case Key_Return:
//          if (contents->playlist->isVisible())
//             clicked(contents->playlist->currentItem());
//          return;

//    case Key_Space:
//    case Key_P:
//    case 0x201c:         // Play/Pause
      case ACTION_PAUSE_RESUME:
         playClicked();
          return true;

//    case Key_Right:
      case ACTION_SEEK_F10:
         fwdClicked();
          return true;

//    case Key_Left:
      case ACTION_SEEK_B10:
         revClicked();
          return true;

//    case Key_0:
//       seek(0, 1);    // begining or end ?
//        return true;

//    case Key_X:
      case ACTION_SEEK_F60:
         seek(60);
          return true;
//    case Key_C:
      case ACTION_SEEK_B60:
         seek(-60);
          return true;

//    case Key_Z:
      case ACTION_SEEK_F600:
         seek(600);
          return true;
//    case Key_V:
      case ACTION_SEEK_B600:
         seek(-600);
          return true;

//    case Key_Up:
//    case 0x201f:         // Backward
      case ACTION_NEXT:
         endClicked();
          return true;
//    case Key_Down:
//    case 0x201e:         // Forward
      case ACTION_PREV:
         startClicked();
          return true;

      // volume up/down, mute
      case ACTION_VOLUME_UP:
         volumeUp();
         return true;

      case ACTION_VOLUME_DOWN:
         volumeDown();
         return true;

      case ACTION_MUTE:
         mute(!muted);
         return true;

//    case Key_Escape:
//    case 0x201d:            // Stop
//    case Key_Return:
//    case Key_Enter:
//    case Key_Q:
      case ACTION_PLAY_STOP:
//          if (isPlaying)
          stop();
          return true;

//    case Key_O:
      case ACTION_OSD:
          sendCommand("osd\n");
          return true;

//    case Key_S:
      case ACTION_SUBTITLES:
          sendCommand("sub_visibility\n");
          return true;

      case ACTION_LCD:
          if (isLCDBlank)
            lcdOn();
          else
            lcdOff();
          return true;

      case ACTION_ADD_TO_PLAYLIST:
          addToPlaylist();
          return true;

      case ACTION_ADD_ALL_TO_PLAYLIST:
          addAllToPlaylist();
          return true;

      case ACTION_REMOVE_FROM_PLAYLIST:
          removeFromPlaylist();
          return true;

      case ACTION_CLEAR_PLAYLIST:
          clearPlaylist();
          return true;

      case ACTION_PLAYLIST_UP:
          contents->playlist->up();
          return true;

      case ACTION_PLAYLIST_DOWN:
          contents->playlist->down();
          return true;

      case ACTION_FILE_INFO:
          displayFileInfo();
          return true;

      default:
         return false;
    }

    return false;
}

void Player::goodBye()
{
    if (kinoPrefs.f_keyboard)
   	 releaseKeyboard();

    close();
}

void Player::focusInEvent(QFocusEvent* /* event */)
{
   if (isPlaying && kinoPrefs.f_keyboard)
   {
      if (kinoPrefs.f_debug)
         qDebug("Grabbing keyboard.");

      grabKeyboard();
   }
}

void Player::focusOutEvent(QFocusEvent* /* event */)
{
   if (isPlaying && kinoPrefs.f_keyboard)
   {
      if (kinoPrefs.f_debug)
         qDebug("Grabbing keyboard.");

      grabKeyboard();
   }
}

void Player::lcdOn()
{
#ifndef QT_NO_COP
    QCopEnvelope e("QPE/System", "setBlankLCD(int)");
    e << 0;
#endif // QT_NO_COP

    if (kinoPrefs.f_debug)
      qDebug("Player::lcdOn()");

	isLCDBlank = false;
}

void Player::lcdOff()
{
#ifndef QT_NO_COP
    QCopEnvelope e("QPE/System", "setBlankLCD(int)");
    e << 1;
#endif // QT_NO_COP

    if (kinoPrefs.f_debug)
      qDebug("Player::lcdOff()");

	isLCDBlank = true;
}

void Player::sysChannelReceived(const QCString& msg, const QByteArray& data)
{
  QDataStream stream(data, IO_ReadOnly);
  if(msg == "setBlankLCD(int)")
  {
    int off;
    stream >> off;

    if (kinoPrefs.f_debug)
      qDebug("Recieved setBlankLCD(int): " + QString::number(off));

    isLCDBlank = (off == 1);
  }
  else if(msg == "rotateChanged(QString)")
  {
      QString newOrientation;
      stream >> newOrientation;

      if (kinoPrefs.f_debug)
         qDebug("Recieved rotateChanged(QString): " + newOrientation);

      if (newOrientation.startsWith("Transformed:Rot0"))
         orientation = Vertical;
      else
         orientation = Horizontal;
  }
  else if(msg == "rotateChangedDone()")
  {
      if (kinoPrefs.f_debug)
         qDebug("Recieved rotateChangedDone()");

      contents->updateLayout(orientation, m_viewMenu->isItemChecked(VIEW_ALBUM_COVER));
  }
}
/*
void Player::setBlankLCD(int off)
{
	if (kinoPrefs.f_debug)
		qDebug("Recieved setBlankLCD(int): " + QString::number(off));

	isLCDBlank = off;
}
*/

Player::ZAURUS_MODEL Player::detectModel()
{
	ZAURUS_MODEL model = UNSUPPORTED;
    QFile f("/proc/deviceinfo/product");
    if (f.open(IO_ReadOnly))
    {
        QTextStream t(&f);        // use a text stream
        while (!t.eof())  			// until end of file...
        {
            QString sModel = t.readLine();       // line of text excluding '\n'
            if (sModel == "SL-5600")
            	model = SL_5600;
            else if (sModel == "SL-6000")
            	model = SL_6000;
            else if (sModel == "SL-C700" || sModel == "SL-C750" || sModel == "SL-C7500" || sModel == "SL-C760" || sModel == "SL-C860")
            	model = SL_C7X0_860;
            else if (sModel == "SL-C3000" || sModel == "SL-C1000" || sModel == "SL-C3100" || sModel == "SL-C3500")
            	model = SL_C3000;
            else
            	model = UNSUPPORTED;
        }
        f.close();
    }

    return model;
}
