/****************************************************************************************
 * (c) Cacko Team,2002,2003								*
 * Kino2 video player									*
 * http://www.cacko.biz									*
 ****************************************************************************************/

#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <qmenubar.h>
#include <qtoolbar.h>
#include <qmainwindow.h>
#include <qlist.h>
#include <qmap.h>
#include <qvbox.h>
#include <qwidgetstack.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qstringlist.h>
#include <qlayout.h>
#include <qslider.h>

#include <sl/slfileselector.h>
#include <sl/sldlgwait.h>

#include <qpe/qcopenvelope_qws.h>

#include "playerbase.h"
#include "oprocess.h"
//#include "control.h"
#include "kinosettings.h"
#include "MediaInfo.h"
#include "playlist.h"
#include "FileSelectionFilter.h"

#define KINO_VERSION    "0.4.3b"
#define MPLAYER            "mplayer"
#define DEFAULT_FILE_TYPES  "*.mp3;*.ogg;*.ogm;*.wma;*.wav;*.asf;*.au;*.avi;*.mpeg;*.mpg;*.mv1;*.mov;*.wmv;*.pls;*.m3u;*.mp4;*.m4a"
#define DEFAULT_PLAYLIST    "/.playlist/Playlist.m3u"

// default maximum value for software volume control, 255 = 100%
#define DEFAULT_MAX_SOFT_VOL    255

// default volume step percent
#define DEFAULT_VOL_STEP 2

class PlayerBaseImpl : public PlayerBase
{
   Q_OBJECT

   public:
    PlayerBaseImpl( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 )
      : PlayerBase(parent, name, fl)
    {}

    ~PlayerBaseImpl() {};
    void updateLayout(Qt::Orientation orientation, bool albumVisible);
};

class Player : public QMainWindow
{
    Q_OBJECT

public:
    Player( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~Player();

    // all player actions
    enum ACTIONS
    {
    	ACTION_NONE,
    	ACTION_PLAY_STOP,
    	ACTION_PAUSE_RESUME,
    	ACTION_VOLUME_UP,
    	ACTION_VOLUME_DOWN,
    	ACTION_MUTE,
    	ACTION_SEEK_F10,
    	ACTION_SEEK_B10,
    	ACTION_SEEK_F60,
    	ACTION_SEEK_B60,
    	ACTION_SEEK_F600,
    	ACTION_SEEK_B600,
    	ACTION_NEXT,
    	ACTION_PREV,
		ACTION_OSD,
		ACTION_SUBTITLES,
		ACTION_ADD_TO_PLAYLIST,
      ACTION_ADD_ALL_TO_PLAYLIST,
		ACTION_REMOVE_FROM_PLAYLIST,
		ACTION_PLAYLIST_UP,
		ACTION_PLAYLIST_DOWN,
		ACTION_FILE_INFO,
      ACTION_LCD,
      ACTION_CLEAR_PLAYLIST,
      ACTION_LAST
    };

    enum FILE_MENU_ITEMS
    {
        FILE_OPEN,
        FILE_OPEN_URL,
        FILE_SAVE_PLAYLIST,
        FILE_SEPARATOR1,
        FILE_INFO,
        FILE_SEPARATOR2,
        FILE_PREFS
    };

    enum VIEW_MENU_ITEMS
    {
        VIEW_SCALE,
        VIEW_LCD_OFF,
        VIEW_SEPARATOR1,
        VIEW_PLAYLIST,
        VIEW_SLIDER,
        VIEW_ALBUM_COVER
    };

    enum ZAURUS_MODEL
    {
    	UNSUPPORTED,
    	SL_5600,
    	SL_6000,
    	SL_C7X0_860,
    	SL_C3000
    };

private:
//    Control	*fscr_control;
    SlFileSelector	*slf;
    FileSelectionFilter* filter;

    QToolBar	*bar;
    QMenuBar	*menu;
    QPopupMenu	*m_fileMenu;
    QPopupMenu	*m_viewMenu;
    QPopupMenu	*m_helpMenu;
    QPopupMenu  *m_columnsMenu;

    QToolButton	*addToPls;
    QToolButton   *addAllToPls;
    QToolButton	*delFromPls;
    QToolButton	*clearPls;
    QToolButton	*info;

    QToolButton	*up;
    QToolButton	*down;

    QToolButton	*columns;

    QWidgetStack *stack;
    PlayerBaseImpl	*contents;
    SlDlgWait* wait;
    //PlaylistView *playlist;

    QStringList coverImages;
    unsigned int imageIndex;

    bool	useScale;
    bool	isPlaying;
    bool	isPausing;
    bool	isIdentify;
    bool	isAddToPlaylist;
    bool	isManualStop;
    bool	isRestartPlaying;
    bool	isLCDBlank;
    bool	isAutoplay;
    bool	isVideoFile;
    bool isIgnoreOutput;
    bool	isSelectFirst;

    bool doubleKey;
    int savedKey;

    bool closeFileSelector;
    bool needToRefresh;
    bool positionRestored;

    int errorCount;
    unsigned errorTimeout;

    ZAURUS_MODEL m_model;
    bool warningDisplayed;

    QString bufferUtf8;
    QString bufferLocal;

    QString		selectedFile;
    QString    saveCurrentFile;
    int			total_time;
    int        playlist_pos;
    int        file_pos;
    bool       saved_lid_suspend;

    OProcess*	process;
    QColor		old_bg_color;
    QPixmap* 	splash;

    KinoPrefs	kinoPrefs;

    ClockVCore freq;

    typedef QMap<QString, MediaInfo> MediaInfoMap;
    MediaInfoMap	minfo;

	 QCopChannel *channel, *sysChannel;
	 int intervalDim, intervalLightOff, intervalSuspend, dim;

    Qt::Orientation orientation;

    void setSelectedFile(const QString& selectedFile);
    void loadSettings();
    void saveSettings();

    void loadRecentFiles();
    void saveRecentFiles();

    bool addUrl(const QString& url);
    bool addFile(const QFileInfo& fileInfo);
    bool addFolder(const QFileInfo& fileInfo);
    bool addPlaylist(const QFileInfo& fileInfo);
    void savePlaylist(const QString& file);

    void identifyFile(const QString& file);
    void identifyFile(const QStringList& files);

    QStringList loadCoverImages(const QDir& folder);

    void parseMediaInfo(const QString& file, const QString& strUtf8, const QString& strLocal);

	void toggleSuspend(bool enable);
   void toggleLidSuspend(bool disable);

	void showPlaylist(bool visible);
	void showSlider(bool visible);
   void showAlbumCover(bool visible);

   void displayCoverImage();

	void sendCommand(const QString& str);
	void lcdOn();
	void lcdOff();
	void setupButtons();
	void addPlayitem(QStringList& args, QString file);

	void seek(int value, int type = 0);
	void pause();
	void resume();
	void stop();

	void volumeUp();
	void volumeDown();
	int readSystemVolume();
	void writeSystemVolume(int volume);

	void setVolume(int percent);
   void setSoftwareVolume(int percent);
	void mute(bool toggled);

   void fadeOut();
   void fadeIn();

	ZAURUS_MODEL detectModel();

	bool muted;
	int volumePercent;
   int sysVolume;
   int volumeFade;
   int volumeStep;
	int currentPosition;
   int jumpTo;

   Player::ACTIONS lookupActionForKey(int key);
   QString length(int seconds) const { return  QString().sprintf("%02d:%02d", seconds / 60, seconds % 60); };

public slots:
   void restoreSysVolume();

   void fadeOutStep();
   void fadeInStep();

   void delayedPause();
   void delayedStop();

   void delayedPrev();
   void delayedNext();

   void displayFileInfo();
   void closeEvent(QCloseEvent *e);
	void keyPressed(QKeyEvent *, bool&);
	void loadDefaultPlaylist();

//   void setDocument(const QCString& msg, const QByteArray& data);
   void sysChannelReceived(const QCString& msg, const QByteArray& data);
   void setDocument(const QString&);
//   void setBlankLCD(int);

   void volumeChanged(bool nowMuted);

protected:
    void keyPressEvent(QKeyEvent *event);
    bool processKey(int key);
    void focusInEvent(QFocusEvent *event);
    void focusOutEvent(QFocusEvent *event);

private slots:
    void fileMenu(int item);
    void viewMenu(int item);
    void helpMenu(int item);
    void columnsMenu(int item);
    void startClicked(void);
    void endClicked(void);
    void playClicked(void);
    void ejectClicked(void);
    void stopClicked(void);
    void revClicked(void);
    void fwdClicked(void);

//    void done(int status);
//    void block_in(const char* s, int len);

	void processExited(OProcess *proc);
  	void receivedStdout(OProcess *proc, char *buffer, int buflen);
  	void receivedStderr(OProcess *proc, char *buffer, int buflen);
  	void wroteStdin(OProcess *proc);

    bool addToPlaylist();
    bool addAllToPlaylist();
    bool addToPlaylist(const QFileInfo& fileInfo);
    void removeFromPlaylist();
    void clearPlaylist();
    void currentChanged(QListViewItem* item);
    void clicked(QListViewItem* item);

    void sliderPressed();
    void sliderMoved(int value);
    void sliderReleased();

    void doubleKeyTimeout();

protected slots:
    void goodBye();
    void fileSelected(const QFileInfo &fs);
};

#endif // _PLAYER_H_
